# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from datetime import timedelta
from flask import current_app as app
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utc import utcnow
from superdesk.utils import get_random_string
from superdesk.emails import send_reset_password_email
from superdesk.errors import SuperdeskApiError
from flask_babel import _

logger = logging.getLogger(__name__)


reset_schema = {
    "email": {"type": "email"},
    "token": {"type": "string"},
    "password": {"type": "string"},
    "expire_time": {"type": "datetime"},
    "user": Resource.rel("users", True),
}


class ActiveTokensResource(Resource):
    internal_resource = True
    schema = reset_schema
    where_clause = "this.expire_time >= ISODate()"
    datasource = {
        "source": "reset_user_password",
        "default_sort": [("_created", -1)],
        "filter": {"$where": where_clause},
    }
    resource_methods = []
    item_methods = []


class ResetPasswordResource(Resource):
    schema = reset_schema
    public_methods = ["POST"]
    resource_methods = ["POST"]
    item_methods = []


class ResetPasswordService(BaseService):
    def check_if_valid_token(self, token):
        reset_request = superdesk.get_resource_service("active_tokens").find_one(req=None, token=token)
        if not reset_request:
            logger.warning("Invalid token received: %s" % token)
            raise SuperdeskApiError.unauthorizedError("Invalid token received")

        return reset_request

    def create(self, docs, **kwargs):
        for doc in docs:
            email = doc.get("email")
            key = doc.get("token")
            password = doc.get("password")

            if key and password:
                return self.reset_password(doc)
            if email:
                email = email.lower()
                return self.initialize_reset_password(doc, email)
            if key:
                token_req = self.check_if_valid_token(key)
                return [token_req.get("_id")]

            raise SuperdeskApiError.badRequestError(_("Either key:password or email must be provided"))

    def store_reset_password_token(self, doc, email, days_alive, user_id):
        now = utcnow()
        doc[app.config["DATE_CREATED"]] = now
        doc[app.config["LAST_UPDATED"]] = now
        doc["expire_time"] = now + timedelta(days=days_alive)
        doc["user"] = user_id
        doc["token"] = get_random_string()
        ids = super().create([doc])
        return ids

    def initialize_reset_password(self, doc, email):
        token_ttl = app.config["RESET_PASSWORD_TOKEN_TIME_TO_LIVE"]

        user = superdesk.get_resource_service("users").find_one(req=None, email=email)
        if not user:
            logger.warning("User password reset triggered with invalid email: %s" % email)
            raise SuperdeskApiError.badRequestError(_("Invalid email"))

        if not user.get("is_enabled", False):
            logger.warning("User password reset triggered for an disabled user")
            raise SuperdeskApiError.forbiddenError(_("User not enabled"))

        if not user.get("is_active", False):
            logger.warning("User password reset triggered for an inactive user")
            raise SuperdeskApiError.forbiddenError(_("User not active"))

        ids = self.store_reset_password_token(doc, email, token_ttl, user["_id"])
        send_reset_password_email(doc, token_ttl)
        self.remove_private_data(doc)
        return ids

    def reset_password(self, doc):
        key = doc.get("token")
        password = doc.get("password")

        reset_request = self.check_if_valid_token(key)

        user_id = reset_request["user"]
        user = superdesk.get_resource_service("users").find_one(req=None, _id=user_id)
        if not user.get("is_active"):
            logger.warning("Try to set password for an inactive user")
            raise SuperdeskApiError.forbiddenError(_("User not active"))

        superdesk.get_resource_service("users").update_password(user_id, password)
        self.remove_all_tokens_for_email(reset_request["email"])
        self.remove_private_data(doc)
        return [reset_request["_id"]]

    def remove_all_tokens_for_email(self, email):
        super().delete(lookup={"email": email})

    def remove_private_data(self, doc):
        self.remove_field_from(doc, "password")
        self.remove_field_from(doc, "token")
        self.remove_field_from(doc, "user")

    def remove_field_from(self, doc, field_name):
        if doc and doc.get(field_name):
            del doc[field_name]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from apps.search_providers import allowed_search_providers
from eve.utils import config
from superdesk.errors import SuperdeskApiError
from superdesk.services import BaseService
from superdesk.utils import ListCursor
from flask_babel import _

logger = logging.getLogger(__name__)


class SearchProviderService(BaseService):
    def get(self, req, lookup):
        """
        Overriding to filter out the providers if they haven't been registered with the application.
        """

        providers = list(super().get(req, lookup))
        filtered_providers = []

        for provider in providers:
            if provider["search_provider"] in allowed_search_providers:
                filtered_providers.append(provider)

        return ListCursor(filtered_providers)

    def find_one(self, req, **lookup):
        """
        Overriding to filter out the providers if they haven't been registered with the application.
        """

        provider = super().find_one(req, **lookup)
        return provider if provider and provider["search_provider"] in allowed_search_providers else None

    def on_created(self, docs):
        for doc in docs:
            if doc.get("is_default"):
                self.find_and_modify(
                    query={"$and": [{"_id": {"$ne": doc[config.ID_FIELD]}}, {"is_default": True}]},
                    update={"$set": {"is_default": False}},
                    upsert=False,
                )

    def on_updated(self, updates, original):
        if updates.get("is_default"):
            self.find_and_modify(
                query={"$and": [{"_id": {"$ne": original[config.ID_FIELD]}}, {"is_default": True}]},
                update={"$set": {"is_default": False}},
                upsert=False,
            )

    def on_delete(self, doc):
        """
        Overriding to check if the search provider being requested to delete has been used to fetch items.
        """

        if doc.get("last_item_update"):
            raise SuperdeskApiError.forbiddenError(_("Deleting a Search Provider after receiving items is prohibited."))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import ssl
from abc import ABCMeta
from datetime import timedelta

import arrow
import requests
from eve.utils import config
from requests.packages.urllib3.poolmanager import PoolManager

from superdesk import get_resource_service
from superdesk.errors import IngestApiError
from superdesk.io.feeding_services import FeedingService
from superdesk.utc import utcnow
from superdesk.etree import etree


class HTTPFeedingService(FeedingService, metaclass=ABCMeta):
    """
    Feeding Service class which can read article(s) using HTTP.
    """

    ERRORS = [
        IngestApiError.apiTimeoutError().get_error_description(),
        IngestApiError.apiRedirectError().get_error_description(),
        IngestApiError.apiRequestError().get_error_description(),
        IngestApiError.apiUnicodeError().get_error_description(),
        IngestApiError.apiParseError().get_error_description(),
        IngestApiError.apiGeneralError().get_error_description(),
    ]

    label = "HTTP"

    def __init__(self):
        super().__init__()
        self.token = None

    def _generate_token_and_update_provider(self, provider):
        """
        Generates Authentication Token and updates the given provider with the authentication token.

        :param provider: dict - Ingest provider details to which the current directory has been configured
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :return: Authentication Token
        :rtype: str
        """
        token = {"auth_token": self._generate_auth_token(provider), "created": utcnow()}
        get_resource_service("ingest_providers").system_update(
            provider[config.ID_FIELD], updates={"tokens": token}, original=provider
        )
        provider["tokens"] = token
        return token["auth_token"]

    def _generate_auth_token(self, provider):
        """
        Generates Authentication Token as per the configuration in Ingest Provider.

        :param provider: dict - Ingest provider details to which the current directory has been configured
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :return: token details if successfully authenticated
        :rtype: str
        :raises: IngestApiError.apiGeneralError() if auth_url is missing in the Ingest Provider configuration
        """
        session = requests.Session()
        session.mount("https://", SSLAdapter())

        auth_url = provider.get("config", {}).get("auth_url", None)
        if not auth_url:
            raise IngestApiError.apiGeneralError(
                provider=provider,
                exception=KeyError(
                    """
                                                     Ingest Provider {} is missing Authentication URL.
                                                     Please check the configuration.
                                                     """.format(
                        provider["name"]
                    )
                ),
            )

        payload = {
            "username": provider.get("config", {}).get("username", ""),
            "password": provider.get("config", {}).get("password", ""),
        }

        response = session.get(auth_url, params=payload, verify=False, timeout=30)
        if response.status_code < 200 or response.status_code >= 300:
            try:
                response.raise_for_status()
            except Exception:
                err = IngestApiError.apiAuthError(provider=provider)
                self.close_provider(provider, err, force=True)
                raise err

        tree = etree.fromstring(response.content)  # workaround for http mock lib
        return tree.text

    def _is_valid_token(self, token):
        """Check if the given token is still valid.

        Most of authentication tokens issued by Ingest Providers are valid for 12 hours.

        :param token: Token information
        :type token: dict
        :return: True if valid, False otherwise
        :rtype: bool
        """
        ttl = timedelta(hours=12)
        created = arrow.get(token.get("created")).datetime

        return created + ttl >= utcnow() and token.get("auth_token")

    def _get_auth_token(self, provider, update=False):
        """
        Gets authentication token for given provider instance and save it in db based on the given update flag.

        :param provider: dict - Ingest provider details to which the current directory has been configured
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :param update: a flag which dictates whether to save the authentication token in Ingest Provider record or not.
                       Saves if the value is True, defaults to False.
        :type update: bool
        :return: Authentication Token
        :rtype: str
        """
        token = provider.get("tokens")

        if token and self._is_valid_token(token):
            return token.get("auth_token")

        return self._generate_token_and_update_provider(provider) if update else ""


# workaround for ssl version error
class SSLAdapter(requests.adapters.HTTPAdapter):
    """
    SSL Adapter set for ssl tls v1.
    """

    def init_poolmanager(self, connections, maxsize, **kwargs):
        """
        Initializes pool manager to use ssl version v1.
        """

        self.poolmanager = PoolManager(num_pools=connections, maxsize=maxsize, ssl_version=ssl.PROTOCOL_TLSv1, **kwargs)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import request
from eve.utils import config
from eve.methods.common import resolve_document_etag
import logging
import superdesk
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utc import utcnow
from superdesk import get_backend
from superdesk import get_resource_service
from superdesk.workflow import get_privileged_actions
from superdesk.validation import ValidationError
from flask_babel import _


_preferences_key = "preferences"
_user_preferences_key = "user_preferences"
_session_preferences_key = "session_preferences"
_privileges_key = "active_privileges"
_action_key = "allowed_actions"
logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = "preferences"
    service = PreferencesService(endpoint_name, backend=get_backend())
    PreferencesResource(endpoint_name, app=app, service=service)
    app.on_session_end -= service.on_session_end
    app.on_session_end += service.on_session_end
    app.on_role_privileges_updated -= service.on_role_privileges_updated
    app.on_role_privileges_updated += service.on_role_privileges_updated
    superdesk.intrinsic_privilege(resource_name=endpoint_name, method=["PATCH"])


def enhance_document_with_default_prefs(doc):
    user_prefs = doc.get(_user_preferences_key, {})
    available = dict(superdesk.default_user_preferences)
    available.update(user_prefs)

    def sync_field(field, dest, default):
        if not isinstance(dest, dict) or not isinstance(default, dict):
            return
        if default.get(field):
            dest[field] = default[field]
        elif dest.get(field):
            dest.pop(field, None)

    # make sure label and category are up-to-date
    for k, v in available.items():
        default = superdesk.default_user_preferences.get(k)
        if default:
            sync_field("label", v, default)
            sync_field("category", v, default)

        if isinstance(v, dict) and v.get("label"):
            v["label"] = _(v["label"])
        if isinstance(v, dict) and v.get("category"):
            v["category_label"] = _(v["category"])

    doc[_user_preferences_key] = available


class PreferencesResource(Resource):
    datasource = {
        "source": "users",
        "projection": {
            _session_preferences_key: 1,
            _user_preferences_key: 1,
            _privileges_key: 1,
            _action_key: 1,
            "_etag": 1,
            "role": 1,
            "user_type": 1,
            "privileges": 1,
        },
    }
    schema = {
        _session_preferences_key: {"type": "dict", "required": True, "allow_unknown": True},
        _user_preferences_key: {"type": "dict", "required": True, "allow_unknown": True},
        _privileges_key: {"type": "dict", "allow_unknown": True},
        _action_key: {"type": "list"},
        # we need these to get user/role info
        "role": {"type": "string"},
        "user_type": {"type": "string"},
        "privileges": {"type": "dict"},
    }
    allow_unknown = True
    resource_methods = []
    item_methods = ["GET", "PATCH"]
    merge_nested_documents = True

    superdesk.register_default_user_preference(
        "feature:preview",
        {
            "type": "bool",
            "enabled": False,
            "default": False,
            "label": _("Enable Feature Preview"),
            "category": _("feature"),
            "privileges": ["feature_preview"],
        },
    )

    superdesk.register_default_user_preference(
        "archive:view",
        {
            "type": "string",
            "allowed": ["mgrid", "compact"],
            "view": "mgrid",
            "default": "mgrid",
            "label": _("Users archive view format"),
            "category": _("archive"),
        },
    )

    superdesk.register_default_user_preference(
        "singleline:view",
        {
            "type": "bool",
            "enabled": None,
            "default": False,
            "label": _("Enable Single Line View"),
            "category": _("rows"),
        },
    )

    superdesk.register_default_user_preference(
        "editor:theme",
        {
            "type": "string",
            "theme": "",
        },
    )

    superdesk.register_default_user_preference("workqueue:items", {"items": []})

    superdesk.register_default_user_preference("dashboard:ingest", {"providers": []})

    superdesk.register_default_user_preference(
        "agg:view",
        {
            "active": {},
        },
    )

    superdesk.register_default_user_preference("templates:recent", {})

    superdesk.register_default_user_preference(
        "dateline:located", {"type": "dict", "label": _("Located"), "category": _("article_defaults")}
    )

    superdesk.register_default_user_preference(
        "categories:preferred",
        {
            "type": "dict",
            "category": _("categories"),
            "label": _("Preferred Categories"),
            "selected": {},
        },
    )

    superdesk.register_default_user_preference(
        "desks:preferred",
        {
            "type": "dict",
            "category": _("desks"),
            "label": _("Preferred Desks"),
            "selected": {},
        },
    )

    superdesk.register_default_user_preference(
        "article:default:place", {"type": "list", "label": _("Place"), "category": _("article_defaults"), "place": []}
    )

    superdesk.register_default_user_preference(
        "spellchecker:status", {"type": "bool", "enabled": True, "default": True}
    )

    superdesk.register_default_user_preference(
        "contacts:view",
        {
            "type": "string",
            "allowed": ["mgrid", "compact"],
            "view": "mgrid",
            "default": "mgrid",
            "label": _("Users contacts view format"),
            "category": _("contacts"),
        },
    )

    superdesk.register_default_user_preference("destination:active", {})

    superdesk.register_default_user_preference("extensions", {})

    superdesk.register_default_user_preference(
        "search:filters_panel_open", {"type": "bool", "enabled": True, "default": True}
    )

    superdesk.register_default_user_preference("masterdesk:desks", {})

    superdesk.register_default_user_preference("editor:char_limit_ui", {})

    superdesk.register_default_session_preference("scratchpad:items", [])
    superdesk.register_default_session_preference("desk:last_worked", "")
    superdesk.register_default_session_preference("desk:items", [])
    superdesk.register_default_session_preference("stage:items", [])
    superdesk.register_default_session_preference("pinned:items", [])


class PreferencesService(BaseService):
    def on_session_end(self, user_id, session_id):
        service = get_resource_service("users")
        user_doc = service.find_one(req=None, _id=user_id)
        session_prefs = user_doc.get(_session_preferences_key, {}).copy()

        if not isinstance(session_id, str):
            session_id = str(session_id)

        if session_id in session_prefs:
            del session_prefs[session_id]
            service.system_update(user_id, {_session_preferences_key: session_prefs}, user_doc)

    def set_session_based_prefs(self, session_id, user_id):
        service = get_resource_service("users")
        user_doc = service.find_one(req=None, _id=user_id)

        session_prefs = user_doc.get(_session_preferences_key, {})
        available = dict(superdesk.default_session_preferences)
        if available.get("desk:last_worked") == "" and user_doc.get("desk"):
            available["desk:last_worked"] = user_doc.get("desk")

        session_prefs.setdefault(str(session_id), available)
        service.system_update(user_id, {_session_preferences_key: session_prefs}, user_doc)

    def set_user_initial_prefs(self, user_doc):
        if _user_preferences_key not in user_doc:
            orig_user_prefs = user_doc.get(_preferences_key, {})
            available = dict(superdesk.default_user_preferences)
            available.update(orig_user_prefs)
            user_doc[_user_preferences_key] = available

    def find_one(self, req, **lookup):
        session = get_resource_service("sessions").find_one(req=None, _id=lookup["_id"])
        _id = session["user"] if session else lookup["_id"]
        doc = get_resource_service("users").find_one(req, _id=_id)
        if doc:
            doc["_id"] = session["_id"] if session else _id
        return doc

    def on_fetched_item(self, doc):
        session_id = request.view_args["_id"]
        session_prefs = doc.get(_session_preferences_key, {}).get(session_id, {})
        doc[_session_preferences_key] = session_prefs
        self.enhance_document_with_user_privileges(doc)
        enhance_document_with_default_prefs(doc)
        self._filter_preferences_by_privileges(doc)

    def on_update(self, updates, original):
        existing_user_preferences = original.get(_user_preferences_key, {}).copy()
        existing_session_preferences = original.get(_session_preferences_key, {}).copy()

        self.update_user_prefs(updates, existing_user_preferences)
        session_id = request.view_args["_id"]
        self.update_session_prefs(updates, existing_session_preferences, session_id)

    def update_session_prefs(self, updates, existing_session_preferences, session_id):
        session_prefs = updates.get(_session_preferences_key)
        if session_prefs is not None:
            for k in (k for k, v in session_prefs.items() if k not in superdesk.default_session_preferences):
                raise ValidationError(_("Invalid preference: {preference}").format(preference=k))

            existing = existing_session_preferences.get(session_id, {})
            existing.update(session_prefs)
            existing_session_preferences[session_id] = existing
            updates[_session_preferences_key] = existing_session_preferences

    def update_user_prefs(self, updates, existing_user_preferences):
        user_prefs = updates.get(_user_preferences_key)
        if user_prefs is not None:
            # check if the input is validated against the default values
            for k in (k for k, v in user_prefs.items() if k not in superdesk.default_user_preferences):
                raise ValidationError(_("Invalid preference: {preference}").format(preference=k))

            existing_user_preferences.update(user_prefs)
            updates[_user_preferences_key] = existing_user_preferences

    def update(self, id, updates, original):
        session = get_resource_service("sessions").find_one(req=None, _id=original["_id"])
        original_unpatched = self.backend.find_one(self.datasource, req=None, _id=session["user"])
        updated = original_unpatched.copy()
        updated.update(updates)
        del updated["_id"]
        res = self.backend.update(self.datasource, original_unpatched["_id"], updated, original_unpatched)
        updates.update(updated)
        # Return only the patched session prefs
        session_prefs = updates.get(_session_preferences_key, {}).get(str(original["_id"]), {})
        updates[_session_preferences_key] = session_prefs
        self.enhance_document_with_user_privileges(updates)
        enhance_document_with_default_prefs(updates)
        return res

    def enhance_document_with_user_privileges(self, user_doc):
        role_doc = get_resource_service("users").get_role(user_doc)
        get_resource_service("users").set_privileges(user_doc, role_doc)
        user_doc[_action_key] = get_privileged_actions(user_doc[_privileges_key])

    def get_user_preference(self, user_id):
        """
        This function returns preferences for the user.
        """
        doc = get_resource_service("users").find_one(req=None, _id=user_id)
        prefs = doc.get(_user_preferences_key, {})
        return prefs

    def email_notification_is_enabled(self, user_id=None, preferences=None):
        """
        This function checks if email notification is enabled or not based on the preferences.
        """
        if user_id:
            preferences = self.get_user_preference(user_id)
        send_email = preferences.get("email:notification", {}) if isinstance(preferences, dict) else {}
        return send_email and send_email.get("enabled", False)

    def is_authorized(self, **kwargs):
        """
        Returns False if logged-in user is trying to update other user's or session's privileges.

        :param kwargs:
        :return: True if authorized, False otherwise
        """
        if not kwargs.get("_id") or not kwargs.get("user_id"):
            return False

        session = get_resource_service("sessions").find_one(req=None, _id=kwargs.get("_id"))
        if not session:
            return False

        return str(kwargs.get("user_id")) == str(session.get("user"))

    def on_role_privileges_updated(self, role, role_users):
        """Runs when user privilage has been updated.

        Update the session for active user so that preferences can be reloaded.

        :param dict role: role getting updated
        :param list role_users: list of user belonging to the role.
        """
        if not role_users or not role:
            return

        logger.info("On_Role_Privileges_Updated: Updating Users for Role:{}.".format(role.get(config.ID_FIELD)))
        for user in role_users:
            try:
                super().update(user[config.ID_FIELD], {}, user)
            except Exception:
                logger.warn(
                    "On_Role_Privileges_Updated:Failed to update user:{} with role:{}.".format(
                        user.get(config.ID_FIELD), role.get(config.ID_FIELD)
                    ),
                    exc_info=True,
                )

    def _filter_preferences_by_privileges(self, doc):
        privileges = doc[_privileges_key]
        preferences = doc[_user_preferences_key]

        def has_missing_privileges(prefs):
            prefs = prefs if isinstance(prefs, list) else [prefs]
            return [priv for pref in prefs for priv in pref.get("privileges", []) if not privileges.get(priv)]

        doc[_user_preferences_key] = {k: v for k, v in preferences.items() if not has_missing_privileges(v)}



import pytz
import flask
import base64
import unittest
import responses
import superdesk

from datetime import datetime, timedelta
from unittest.mock import patch
from tests.mock import resources
from superdesk.json_utils import SuperdeskJSONEncoder
from superdesk.publish.formatters.imatrics import IMatricsFormatter
from superdesk.publish.transmitters.imatrics import IMatricsTransmitter


class IMatricsTransmitterTestCase(unittest.TestCase):
    def setUp(self):
        self.app = flask.Flask(__name__)
        self.app.config.update(
            {
                "IMATRICS_BASE_URL": "https://webdemo.imatrics.com/api/",
                "IMATRICS_USER": "foo",
                "IMATRICS_KEY": "key",
            }
        )

    @responses.activate
    def test_publish_article(self):
        start = datetime(2020, 10, 8, 10, 0, 0, tzinfo=pytz.UTC)
        with self.app.app_context():
            with patch.dict(superdesk.resources, resources):
                responses.add(
                    responses.POST,
                    url=self.app.config["IMATRICS_BASE_URL"] + "article/concept",
                    json={"uuid": "guid"},
                )
                formatter = IMatricsFormatter()
                subscriber = {}
                transmitter = IMatricsTransmitter()
                item = {
                    "guid": "guid",
                    "language": "en",
                    "pubstatus": "usable",
                    "abstract": "<p>foo bar baz</p>",
                    "dateline": {"text": "dateline.text"},
                    "firstcreated": start,
                    "versioncreated": start + timedelta(minutes=10),
                    "firstpublished": start + timedelta(minutes=20),
                    "headline": "headline <b>text</b>",
                    "body_html": "<p>foo bar baz</p><p>another one</p>",
                    "authors": [
                        {"sub_label": "john"},
                    ],
                    "subject": [
                        {
                            "name": "The category",
                            "scheme": "mediatopics",
                            "altids": {
                                "imatrics": "1",
                            },
                        },
                        {
                            "name": "A topic",
                            "scheme": "imatrics_topic",
                            "altids": {
                                "imatrics": "2",
                            },
                        },
                    ],
                    "organisation": [
                        {
                            "name": "An organisation",
                            "altids": {
                                "imatrics": "3",
                            },
                        },
                    ],
                    "person": [
                        {
                            "name": "A person",
                            "altids": {
                                "imatrics": "4",
                            },
                        },
                    ],
                    "place": [
                        {
                            "name": "A place",
                            "altids": {
                                "imatrics": "5",
                            },
                        },
                    ],
                    "event": [
                        {
                            "name": "An event",
                            "altids": {
                                "imatrics": "6",
                            },
                        },
                    ],
                    "object": [
                        {
                            "name": "An object",
                            "altids": {
                                "imatrics": "7",
                            },
                        },
                    ],
                }
                queue_item = {"formatted_item": formatter.format(item, subscriber)[0][1]}
                transmitter._transmit(queue_item, subscriber)
                self.assertEqual(len(responses.calls), 1)
                self.assertEqual(
                    "Basic {}".format(base64.b64encode("foo:key".encode()).decode()),
                    responses.calls[0].request.headers["Authorization"],
                )
                self.maxDiff = None
                self.assertEqual(
                    {
                        "uuid": "guid",
                        "language": "en",
                        "pubStatus": True,
                        "createdTimestamp": item["firstcreated"].isoformat(),
                        "latestVersionTimestamp": item["versioncreated"].isoformat(),
                        "publicationTimestamp": item["firstpublished"].isoformat(),
                        "authors": ["john"],
                        "concepts": [
                            {
                                "title": "The category",
                                "type": "category",
                                "uuid": "1",
                            },
                            {
                                "title": "A topic",
                                "type": "topic",
                                "uuid": "2",
                            },
                            {
                                "title": "An organisation",
                                "type": "organisation",
                                "uuid": "3",
                            },
                            {
                                "title": "A person",
                                "type": "person",
                                "uuid": "4",
                            },
                            {
                                "title": "A place",
                                "type": "place",
                                "uuid": "5",
                            },
                            {
                                "title": "An event",
                                "type": "event",
                                "uuid": "6",
                            },
                            {
                                "title": "An object",
                                "type": "object",
                                "uuid": "7",
                            },
                        ],
                        "headline": "headline text",
                        "preamble": "foo bar baz",
                        "dateline": "dateline.text",
                        "body": [
                            "foo bar baz",
                            "another one",
                        ],
                    },
                    flask.json.loads(responses.calls[0].request.body.decode()),
                )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import logging

from superdesk import get_resource_service
from superdesk.services import BaseService
from superdesk.errors import SuperdeskApiError
from superdesk.publish.formatters import get_all_formatters
from superdesk.utils import ListCursor
from superdesk.validation import ValidationError
from apps.publish.content.common import ITEM_PUBLISH
from apps.content_types import apply_schema
from flask_babel import _

logger = logging.getLogger(__name__)


class FormattersService(BaseService):
    def get(self, req, lookup):
        formatters = get_all_formatters()

        if req.args.get("criteria"):
            formatters = (f for f in formatters if getattr(f, req.args.get("criteria")) is True)

        return ListCursor([{"name": type(f).__name__} for f in formatters])

    def _get_formatter(self, name):
        formatters = get_all_formatters()
        return next((f for f in formatters if type(f).__name__ == name), None)

    def _validate(self, doc):
        """Validates the given story for publish action"""
        validate_item = {"act": ITEM_PUBLISH, "type": doc["type"], "validate": doc}
        validation_errors = get_resource_service("validate").validate(validate_item)
        if validation_errors:
            raise ValidationError(validation_errors)

    def create(self, docs, **kwargs):
        service = get_resource_service("archive")
        doc = docs[0]
        formatter_name = doc.get("formatter_name")

        if not formatter_name:
            raise SuperdeskApiError.badRequestError(_("Formatter name not found"))

        formatter = self._get_formatter(formatter_name)

        if not formatter:
            raise SuperdeskApiError.badRequestError(_("Formatter not found"))

        if "article_id" in doc:
            article_id = doc.get("article_id")
            article = service.find_one(req=None, _id=article_id)

            if not article:
                raise SuperdeskApiError.badRequestError(_("Article not found!"))

            try:
                self._validate(article)
                sequence, formatted_doc = formatter.format(apply_schema(article), {"_id": "0"}, None)[0]
                formatted_doc = formatted_doc.replace("''", "'")

                # respond only with the formatted output if output_field is configured
                if hasattr(formatter, "output_field"):
                    formatted_doc = json.loads(formatted_doc)
                    formatted_doc = formatted_doc.get(formatter.output_field, "").replace("''", "'")
            except Exception as ex:
                logger.exception(ex)
                raise SuperdeskApiError.badRequestError(
                    _("Error in formatting article: {exception}").format(exception=str(ex))
                )

            return [{"formatted_doc": formatted_doc}]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
from bson import ObjectId
from superdesk.tests import TestCase
from superdesk.datalayer import SuperdeskJSONEncoder


class DatalayerTestCase(TestCase):
    def test_find_all(self):
        data = {"name": "test", "privileges": {"ingest": 1, "archive": 1, "fetch": 1}}
        superdesk.get_resource_service("roles").post([data])
        self.assertEqual(1, superdesk.get_resource_service("roles").get(req=None, lookup={}).count())

    def test_json_encoder(self):
        _id = ObjectId()
        encoder = SuperdeskJSONEncoder()
        text = encoder.dumps({"_id": _id, "name": "foo", "group": None})
        self.assertIn('"name":"foo"', text)
        self.assertIn('"group":null', text)
        self.assertIn('"_id":"%s"' % (_id,), text)

    def test_find_with_mongo_query(self):
        service = superdesk.get_resource_service("activity")
        service.post(
            [
                {"resource": "foo", "action": "get"},
                {"resource": "bar", "action": "post"},
            ]
        )

        self.assertEqual(1, service.find({"resource": {"$in": ["foo"]}}).count())
        self.assertEqual(1, service.find({}, max_results=1).count(True))

    def test_set_custom_etag_on_create(self):
        service = superdesk.get_resource_service("activity")
        ids = service.post([{"resource": "foo", "action": "get", "_etag": "foo"}])
        item = service.find_one(None, _id=ids[0])
        self.assertEqual("foo", item["_etag"])

    def test_find_one_type(self):
        self.app.data.insert("archive", [{"guid": "foo"}])
        item = self.app.data.find_one("archive", req=None, guid="foo")
        self.assertIsNotNone(item)
        self.assertEqual("archive", item.get("_type"))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from superdesk.metadata.item import get_schema


class WorkqueueResource(superdesk.Resource):
    endpoint_name = "workqueue"
    datasource = {"source": "archive"}
    schema = get_schema()
    item_methods = ["GET"]


class WorkqueueService(superdesk.Service):

    pass



import superdesk

from datetime import timedelta
from superdesk.utc import utcnow
from superdesk.utils import get_random_token
from content_api import MONGO_PREFIX


class SubscriberTokenResource(superdesk.Resource):
    schema = {
        "_id": {"type": "string", "unique": True},
        "expiry": {"readonly": True},
        "expiry_days": {"type": "integer"},
        "subscriber": superdesk.Resource.rel("subscribers", required=True),
    }

    item_url = 'regex(".+")'
    resource_methods = ["GET", "POST"]
    item_methods = ["GET", "DELETE"]
    privileges = {"POST": "subscribers", "DELETE": "subscribers"}

    datasource = {
        "default_sort": [("_created", 1)],
    }

    mongo_prefix = MONGO_PREFIX


class SubscriberTokenService(superdesk.Service):
    def create(self, docs, **kwargs):
        for doc in docs:
            doc["_id"] = get_random_token()
            if doc.get("expiry_days"):
                doc.setdefault("expiry", utcnow() + timedelta(days=doc["expiry_days"]))
        return super().create(docs, **kwargs)



import uuid
from lxml import etree

from .base import BasePackageElement


class Styles(BasePackageElement):
    """
    As Adobe's idml specification explains:
      ``
      The Styles.xml file contains all of the paragraph, character, object, cell, table,
      and table of contents (TOC) styles used in the document.
      ``
    """

    XML_DECLARATION = '<?xml version="1.0" encoding="utf-8" standalone="yes"?>'
    # http://wwwimages.adobe.com/content/dam/acom/en/devnet/indesign/sdk/cs6/idml/idml-specification.pdf
    # page 430
    PARAGRAPHSTYLE_DEFAULTS = {
        "Imported": "false",
        "SplitDocument": "false",
        "EmitCss": "true",
        "StyleUniqueId": "$ID/",
        "IncludeClass": "true",
        "EmptyNestedStyles": "true",
        "EmptyLineStyles": "true",
        "EmptyGrepStyles": "true",
        "FillColor": "Color/Black",
        "FontStyle": "Regular",
        "PointSize": "12",
        "HorizontalScale": "100",
        "KerningMethod": "$ID/Metrics",
        "Ligatures": "true",
        "PageNumberType": "AutoPageNumber",
        "StrokeWeight": "1",
        "Tracking": "0",
        "Composer": "HL Composer",
        "DropCapCharacters": "0",
        "DropCapLines": "0",
        "BaselineShift": "0",
        "Capitalization": "Normal",
        "StrokeColor": "Swatch/None",
        "HyphenateLadderLimit": "3",
        "VerticalScale": "100",
        "LeftIndent": "0",
        "RightIndent": "0",
        "FirstLineIndent": "0",
        "AutoLeading": "120",
        "AppliedLanguage": "$ID/English: USA",
        "Hyphenation": "true",
        "HyphenateAfterFirst": "2",
        "HyphenateBeforeLast": "2",
        "HyphenateCapitalizedWords": "true",
        "HyphenateWordsLongerThan": "5",
        "NoBreak": "false",
        "HyphenationZone": "36",
        "SpaceBefore": "0",
        "SpaceAfter": "0",
        "Underline": "false",
        "OTFFigureStyle": "Default",
        "DesiredWordSpacing": "100",
        "MaximumWordSpacing": "133",
        "MinimumWordSpacing": "80",
        "DesiredLetterSpacing": "0",
        "MaximumLetterSpacing": "0",
        "MinimumLetterSpacing": "0",
        "DesiredGlyphScaling": "100",
        "MaximumGlyphScaling": "100",
        "MinimumGlyphScaling": "100",
        "StartParagraph": "Anywhere",
        "KeepAllLinesTogether": "false",
        "KeepWithNext": "0",
        "KeepFirstLines": "2",
        "KeepLastLines": "2",
        "Position": "Normal",
        "StrikeThru": "false",
        "CharacterAlignment": "AlignEmCenter",
        "KeepLinesTogether": "false",
        "StrokeTint": "-1",
        "FillTint": "-1",
        "OverprintStroke": "false",
        "OverprintFill": "false",
        "GradientStrokeAngle": "0",
        "GradientFillAngle": "0",
        "GradientStrokeLength": "-1",
        "GradientFillLength": "-1",
        "GradientStrokeStart": "0 0",
        "GradientFillStart": "0 0",
        "Skew": "0",
        "RuleAboveLineWeight": "1",
        "RuleAboveTint": "-1",
        "RuleAboveOffset": "0",
        "RuleAboveLeftIndent": "0",
        "RuleAboveRightIndent": "0",
        "RuleAboveWidth": "ColumnWidth",
        "RuleBelowLineWeight": "1",
        "RuleBelowTint": "-1",
        "RuleBelowOffset": "0",
        "RuleBelowLeftIndent": "0",
        "RuleBelowRightIndent": "0",
        "RuleBelowWidth": "ColumnWidth",
        "RuleAboveOverprint": "false",
        "RuleBelowOverprint": "false",
        "RuleAbove": "false",
        "RuleBelow": "false",
        "LastLineIndent": "0",
        "HyphenateLastWord": "true",
        "ParagraphBreakType": "Anywhere",
        "SingleWordJustification": "FullyJustified",
        "OTFOrdinal": "false",
        "OTFFraction": "false",
        "OTFDiscretionaryLigature": "false",
        "OTFTitling": "false",
        "RuleAboveGapTint": "-1",
        "RuleAboveGapOverprint": "false",
        "RuleBelowGapTint": "-1",
        "RuleBelowGapOverprint": "false",
        "Justification": "LeftAlign",
        "DropcapDetail": "1",
        "PositionalForm": "None",
        "OTFMark": "true",
        "HyphenWeight": "5",
        "OTFLocale": "true",
        "HyphenateAcrossColumns": "true",
        "KeepRuleAboveInFrame": "false",
        "IgnoreEdgeAlignment": "false",
        "OTFSlashedZero": "false",
        "OTFStylisticSets": "0",
        "OTFHistorical": "false",
        "OTFContextualAlternate": "true",
        "UnderlineGapOverprint": "false",
        "UnderlineGapTint": "-1",
        "UnderlineOffset": "-9999",
        "UnderlineOverprint": "false",
        "UnderlineTint": "-1",
        "UnderlineWeight": "-9999",
        "StrikeThroughGapOverprint": "false",
        "StrikeThroughGapTint": "-1",
        "StrikeThroughOffset": "-9999",
        "StrikeThroughOverprint": "false",
        "StrikeThroughTint": "-1",
        "StrikeThroughWeight": "-9999",
        "MiterLimit": "4",
        "StrokeAlignment": "OutsideAlignment",
        "EndJoin": "MiterEndJoin",
        "SpanColumnType": "SingleColumn",
        "SplitColumnInsideGutter": "6",
        "SplitColumnOutsideGutter": "0",
        "KeepWithPrevious": "false",
        "SpanColumnMinSpaceBefore": "0",
        "SpanColumnMinSpaceAfter": "0",
        "OTFSwash": "false",
        "ParagraphShadingTint": "-1",
        "ParagraphShadingOverprint": "false",
        "ParagraphShadingWidth": "ColumnWidth",
        "ParagraphShadingOn": "false",
        "ParagraphShadingClipToFrame": "false",
        "ParagraphShadingSuppressPrinting": "false",
        "ParagraphShadingLeftOffset": "0",
        "ParagraphShadingRightOffset": "0",
        "ParagraphShadingTopOffset": "0",
        "ParagraphShadingBottomOffset": "0",
        "ParagraphShadingTopOrigin": "AscentTopOrigin",
        "ParagraphShadingBottomOrigin": "DescentBottomOrigin",
        "ParagraphBorderTint": "-1",
        "ParagraphBorderOverprint": "false",
        "ParagraphBorderOn": "false",
        "ParagraphBorderGapTint": "-1",
        "ParagraphBorderGapOverprint": "false",
        "Tsume": "0",
        "LeadingAki": "-1",
        "TrailingAki": "-1",
        "KinsokuType": "KinsokuPushInFirst",
        "KinsokuHangType": "None",
        "BunriKinshi": "true",
        "RubyOpenTypePro": "true",
        "RubyFontSize": "-1",
        "RubyAlignment": "RubyJIS",
        "RubyType": "PerCharacterRuby",
        "RubyParentSpacing": "RubyParent121Aki",
        "RubyXScale": "100",
        "RubyYScale": "100",
        "RubyXOffset": "0",
        "RubyYOffset": "0",
        "RubyPosition": "AboveRight",
        "RubyAutoAlign": "true",
        "RubyParentOverhangAmount": "RubyOverhangOneRuby",
        "RubyOverhang": "false",
        "RubyAutoScaling": "false",
        "RubyParentScalingPercent": "66",
        "RubyTint": "-1",
        "RubyOverprintFill": "Auto",
        "RubyStrokeTint": "-1",
        "RubyOverprintStroke": "Auto",
        "RubyWeight": "-1",
        "KentenKind": "None",
        "KentenFontSize": "-1",
        "KentenXScale": "100",
        "KentenYScale": "100",
        "KentenPlacement": "0",
        "KentenAlignment": "AlignKentenCenter",
        "KentenPosition": "AboveRight",
        "KentenCustomCharacter": "",
        "KentenCharacterSet": "CharacterInput",
        "KentenTint": "-1",
        "KentenOverprintFill": "Auto",
        "KentenStrokeTint": "-1",
        "KentenOverprintStroke": "Auto",
        "KentenWeight": "-1",
        "Tatechuyoko": "false",
        "TatechuyokoXOffset": "0",
        "TatechuyokoYOffset": "0",
        "AutoTcy": "0",
        "AutoTcyIncludeRoman": "false",
        "Jidori": "0",
        "GridGyoudori": "0",
        "GridAlignFirstLineOnly": "false",
        "GridAlignment": "None",
        "CharacterRotation": "0",
        "RotateSingleByteCharacters": "false",
        "Rensuuji": "true",
        "ShataiMagnification": "0",
        "ShataiDegreeAngle": "4500",
        "ShataiAdjustTsume": "true",
        "ShataiAdjustRotation": "false",
        "Warichu": "false",
        "WarichuLines": "2",
        "WarichuSize": "50",
        "WarichuLineSpacing": "0",
        "WarichuAlignment": "Auto",
        "WarichuCharsBeforeBreak": "2",
        "WarichuCharsAfterBreak": "2",
        "OTFHVKana": "false",
        "OTFProportionalMetrics": "false",
        "OTFRomanItalics": "false",
        "LeadingModel": "LeadingModelAkiBelow",
        "ScaleAffectsLineHeight": "false",
        "ParagraphGyoudori": "false",
        "CjkGridTracking": "false",
        "GlyphForm": "None",
        "RubyAutoTcyDigits": "0",
        "RubyAutoTcyIncludeRoman": "false",
        "RubyAutoTcyAutoScale": "true",
        "TreatIdeographicSpaceAsSpace": "true",
        "AllowArbitraryHyphenation": "false",
        "BulletsAndNumberingListType": "NoList",
        "NumberingStartAt": "1",
        "NumberingLevel": "1",
        "NumberingContinue": "true",
        "NumberingApplyRestartPolicy": "true",
        "BulletsAlignment": "LeftAlign",
        "NumberingAlignment": "LeftAlign",
        "NumberingExpression": "^#.^t",
        "BulletsTextAfter": "^t",
        "ParagraphBorderLeftOffset": "0",
        "ParagraphBorderRightOffset": "0",
        "ParagraphBorderTopOffset": "0",
        "ParagraphBorderBottomOffset": "0",
        "ParagraphBorderStrokeEndJoin": "MiterEndJoin",
        "ParagraphBorderTopLeftCornerOption": "None",
        "ParagraphBorderTopRightCornerOption": "None",
        "ParagraphBorderBottomLeftCornerOption": "None",
        "ParagraphBorderBottomRightCornerOption": "None",
        "ParagraphBorderTopLeftCornerRadius": "1",
        "ParagraphBorderTopRightCornerRadius": "1",
        "ParagraphBorderBottomLeftCornerRadius": "1",
        "ParagraphBorderBottomRightCornerRadius": "1",
        "ParagraphShadingTopLeftCornerOption": "None",
        "ParagraphShadingTopRightCornerOption": "None",
        "ParagraphShadingBottomLeftCornerOption": "None",
        "ParagraphShadingBottomRightCornerOption": "None",
        "ParagraphShadingTopLeftCornerRadius": "1",
        "ParagraphShadingTopRightCornerRadius": "1",
        "ParagraphShadingBottomLeftCornerRadius": "1",
        "ParagraphShadingBottomRightCornerRadius": "1",
        "ParagraphBorderStrokeEndCap": "ButtEndCap",
        "ParagraphBorderWidth": "ColumnWidth",
        "ParagraphBorderTopOrigin": "AscentTopOrigin",
        "ParagraphBorderBottomOrigin": "DescentBottomOrigin",
        "ParagraphBorderTopLineWeight": "1",
        "ParagraphBorderBottomLineWeight": "1",
        "ParagraphBorderLeftLineWeight": "1",
        "ParagraphBorderRightLineWeight": "1",
        "ParagraphBorderDisplayIfSplits": "false",
        "MergeConsecutiveParaBorders": "false",
        "ProviderHyphenationStyle": "HyphAll",
        "DigitsType": "DefaultDigits",
        "Kashidas": "DefaultKashidas",
        "DiacriticPosition": "OpentypePositionFromBaseline",
        "CharacterDirection": "DefaultDirection",
        "ParagraphDirection": "LeftToRightDirection",
        "ParagraphJustification": "DefaultJustification",
        "ParagraphKashidaWidth": "2",
        "XOffsetDiacritic": "0",
        "YOffsetDiacritic": "0",
        "OTFOverlapSwash": "false",
        "OTFStylisticAlternate": "false",
        "OTFJustificationAlternate": "false",
        "OTFStretchedAlternate": "false",
        "KeyboardDirection": "DefaultDirection",
    }

    @property
    def filename(self):
        """
        Filename inside IDML package.
        Used as a filename for a file inside zip container.
        :return str: filename
        """
        return "Resources/Styles.xml"

    def _build_etree(self):
        self._etree = etree.Element(etree.QName(self.XMLNS_IDPKG, "Styles"), nsmap={"idPkg": self.XMLNS_IDPKG})
        self._etree.set("DOMVersion", self.DOM_VERSION)
        self._add_paragraph_style_group_paragraphs()
        self._add_paragraph_style_group_headings()
        self._add_paragraph_style_group_lists()
        self._add_table_style_group()
        self._add_character_style_group()
        self._add_paragraph_style_group_custom()

    def _add_paragraph_style_group_paragraphs(self):
        """
        Add paragraphs style group
        """
        # RootParagraphStyleGroup
        self._rootparagraphstylegroup = etree.SubElement(
            self._etree,
            "RootParagraphStyleGroup",
            attrib={
                "Self": "root_paragraph_style_group",
            },
        )
        # ParagraphStyleGroup $ID/Paragraphs
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            "ParagraphStyleGroup",
            attrib={"Self": "ParagraphStyleGroup/$ID/Paragraphs", "Name": "$ID/Paragraphs"},
        )
        # ParagraphStyle.Paragraphs:NormalParagraph
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Paragraphs%3aNormalParagraph",
                "Name": "Paragraphs:NormalParagraph",
                "NextStyle": "ParagraphStyle/Paragraphs%3aNormalParagraph",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No paragraph style]"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "12"
        # ParagraphStyle.Paragraphs:Blockquote
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Paragraphs%3aBlockquote",
                "Name": "Paragraphs:Blockquote",
                "NextStyle": "ParagraphStyle/Paragraphs%3aBlockquote",
                "KeyboardShortcut": "0 0",
                "FontStyle": "Italic",
                "LeftIndent": "30",
                "RightIndent": "30",
                "ParagraphBorderOn": "true",
                "ParagraphBorderLeftOffset": "-20",
                "ParagraphBorderTopLineWeight": "0",
                "ParagraphBorderBottomLineWeight": "0",
                "ParagraphBorderLeftLineWeight": "4",
                "ParagraphBorderRightLineWeight": "0",
                "StyleUniqueId": str(uuid.uuid4()),
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No paragraph style]"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "12"
        # ParagraphStyle.Paragraphs:Preformatted
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Paragraphs%3aPreformatted",
                "Name": "Paragraphs:Preformatted",
                "NextStyle": "ParagraphStyle/Paragraphs%3aPreformatted",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No paragraph style]"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "12"
        etree.SubElement(properties, "AppliedFont", attrib={"type": "string"}).text = "Courier"
        # ParagraphStyle.Paragraphs:Preformatted
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Paragraphs%3aNormalParagraph",
                "Name": "Paragraphs:NormalParagraph",
                "NextStyle": "ParagraphStyle/Paragraphs%3aNormalParagraph",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No paragraph style]"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "12"

    def _add_paragraph_style_group_headings(self):
        """
        Add headings style group
        """
        # ParagraphStyleGroup
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            "ParagraphStyleGroup",
            attrib={"Self": "ParagraphStyleGroup/$ID/Headings", "Name": "$ID/Headings"},
        )
        # ParagraphStyle.Headings:Heading1
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Headings%3aHeading1",
                "Name": "Headings:Heading1",
                "NextStyle": "ParagraphStyle/Headings%3aHeading1",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "PointSize": "40",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "40"
        # ParagraphStyle.Headings:Heading2
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Headings%3aHeading2",
                "Name": "Headings:Heading2",
                "NextStyle": "ParagraphStyle/Headings%3aHeading2",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "PointSize": "30",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "30"
        # ParagraphStyle.Headings:Heading3
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Headings%3aHeading3",
                "Name": "Headings:Heading3",
                "NextStyle": "ParagraphStyle/Headings%3aHeading3",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "PointSize": "20",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "20"
        # ParagraphStyle.Headings:Heading4
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Headings%3aHeading4",
                "Name": "Headings:Heading4",
                "NextStyle": "ParagraphStyle/Headings%3aHeading4",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "PointSize": "14",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "14"
        # ParagraphStyle.Headings:Heading5
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Headings%3aHeading5",
                "Name": "Headings:Heading5",
                "NextStyle": "ParagraphStyle/Headings%3aHeading5",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "PointSize": "11",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "11"
        # ParagraphStyle.Headings:Heading6
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Headings%3aHeading6",
                "Name": "Headings:Heading6",
                "NextStyle": "ParagraphStyle/Headings%3aHeading6",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "PointSize": "9",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "9"

    def _add_paragraph_style_group_lists(self):
        """
        Add lists style group
        """
        # ParagraphStyleGroup
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            "ParagraphStyleGroup",
            attrib={"Self": "ParagraphStyleGroup/$ID/Lists", "Name": "$ID/Lists"},
        )
        # ParagraphStyle.Lists:OrderedList
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Lists%3aOrderedList",
                "Name": "Lists:OrderedList",
                "NextStyle": "ParagraphStyle/Lists%3aOrderedList",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No paragraph style]"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "12"
        # ParagraphStyle.Lists:UnorderedList
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Lists%3aUnorderedList",
                "Name": "Lists:UnorderedList",
                "NextStyle": "ParagraphStyle/Lists%3aUnorderedList",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No paragraph style]"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "12"

    def _add_table_style_group(self):
        """
        Add table style group
        """
        # RootTableStyleGroup
        self._roottablestylegroup = etree.SubElement(
            self._etree,
            "RootTableStyleGroup",
            attrib={
                "Self": "root_table_style_group",
            },
        )
        # TableStyle
        tablestyle = etree.SubElement(
            self._roottablestylegroup,
            "TableStyle",
            attrib={
                "Self": "TableStyle/$ID/NormalTable",
                "Name": "$ID/NormalTable",
                "Imported": "false",
                "NextStyle": "TableStyle/$ID/NormalTable",
                "KeyboardShortcut": "0 0",
            },
        )
        # Properties
        properties = etree.SubElement(tablestyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No table style]"

    def _add_character_style_group(self):
        """
        Add characters style group
        """
        # RootCharacterStyleGroup
        self._rootcharacterstylegroup = etree.SubElement(
            self._etree,
            "RootCharacterStyleGroup",
            attrib={
                "Self": "root_character_style_group",
            },
        )
        # CharacterStyle No character style
        etree.SubElement(
            self._rootcharacterstylegroup,
            "CharacterStyle",
            attrib={"Self": "CharacterStyle/$ID/[No character style]", "Name": "$ID/[No character style]"},
        )
        # CharacterStyle.Hyperlink
        characterstyle = etree.SubElement(
            self._rootcharacterstylegroup,
            "CharacterStyle",
            attrib={
                "Self": "CharacterStyle/$ID/Hyperlink",
                "Name": "$ID/Hyperlink",
                "FillColor": "Color/Hyperlink",
                "Underline": "true",
            },
        )
        # Properties
        properties = etree.SubElement(characterstyle, "Properties")
        etree.SubElement(properties, "BasedOn", attrib={"type": "string"}).text = "$ID/[No character style]"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"

    def _add_paragraph_style_group_custom(self):
        """
        Add custom style group
        """
        # ParagraphStyleGroup
        paragraphstylegroup = etree.SubElement(
            self._rootparagraphstylegroup,
            "ParagraphStyleGroup",
            attrib={"Self": "ParagraphStyleGroup/$ID/Custom", "Name": "$ID/Custom"},
        )

        # ParagraphStyle.Custom:Headline
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Custom%3aHeadline",
                "Name": "Custom:Headline",
                "NextStyle": "ParagraphStyle/Custom%3aHeadline",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "FontStyle": "Bold",
                "PointSize": "48",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "48"
        # ParagraphStyle.Custom:Byline
        paragraphstyle = etree.SubElement(
            paragraphstylegroup,
            "ParagraphStyle",
            attrib={
                "Self": "ParagraphStyle/Custom%3aByline",
                "Name": "Custom:Byline",
                "NextStyle": "ParagraphStyle/Custom%3aByline",
                "KeyboardShortcut": "0 0",
                "StyleUniqueId": str(uuid.uuid4()),
                "FontStyle": "Bold",
                "PointSize": "20",
            },
        )
        properties = etree.SubElement(paragraphstyle, "Properties")
        etree.SubElement(
            properties, "BasedOn", attrib={"type": "string"}
        ).text = "ParagraphStyle/$ID/NormalParagraphStyle"
        etree.SubElement(properties, "PreviewColor", attrib={"type": "enumeration"}).text = "Nothing"
        etree.SubElement(properties, "ParagraphShadingColor", attrib={"type": "object"}).text = "Color/Black"
        etree.SubElement(properties, "Leading", attrib={"type": "unit"}).text = "20"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import unittest

from datetime import timedelta
from superdesk.utc import utcnow
from superdesk.etree import etree
from superdesk.text_utils import get_word_count, get_char_count
from superdesk.io.registry import registered_feed_parsers
from superdesk.io.feed_parsers.newsml_1_2 import NewsMLOneFeedParser
from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser
from superdesk.io.feed_parsers.nitf import NITFFeedParser
from superdesk.io.feeding_services.file_service import FileFeedingService


def get_etree(filename):
    dirname = os.path.dirname(os.path.realpath(__file__))
    with open(os.path.join(dirname, "fixtures", filename)) as f:
        return etree.fromstring(f.read().encode("utf-8"))


class UtilsTest(unittest.TestCase):
    def test_get_word_count(self):
        self.assertEqual(2, get_word_count("plain text"), "plain text")
        self.assertEqual(2, get_word_count("<p> html text </p>"), "paragraph")

        self.assertEqual(
            22,
            get_word_count(
                '<doc><p xml:lang="en-US">The weather was superb today in Norfolk, Virginia. Made me want to take\n'
                'out my boat, manufactured by the <org value="acm" idsrc="iptc.org">Acme Boat Company</org>.</p></doc>'
            ),
        )

    def test_get_char_count(self):
        self.assertEqual(10, get_char_count("plain text"))
        self.assertEqual(11, get_char_count("<p> html text </p>"))

        self.assertEqual(
            123,
            get_char_count(
                '<doc><p xml:lang="en-US">The weather was superb today in Norfolk, Virginia. Made me want to take\n'
                'out my boat, manufactured by the <org value="acm" idsrc="iptc.org">Acme Boat Company</org>.</p></doc>'
            ),
        )

    def test_get_xml_parser_newsmlg2(self):
        etree = get_etree("snep.xml")
        self.assertIsInstance(
            FileFeedingService().get_feed_parser({"feed_parser": "newsml2"}, etree), NewsMLTwoFeedParser
        )

    def test_get_xml_parser_nitf(self):
        etree = get_etree("nitf-fishing.xml")
        self.assertIsInstance(FileFeedingService().get_feed_parser({"feed_parser": "nitf"}, etree), NITFFeedParser)

    def test_get_xml_parser_newsml12(self):
        etree = get_etree("afp.xml")
        self.assertIsInstance(
            FileFeedingService().get_feed_parser({"feed_parser": "newsml12"}, etree), NewsMLOneFeedParser
        )

    def test_is_old_content(self):
        service = FileFeedingService()
        self.assertFalse(service.is_old_content(utcnow()))
        self.assertTrue(service.is_old_content(utcnow() - timedelta(minutes=11)))


class ItemTest(unittest.TestCase):
    def setUpFixture(self, filename):
        self.tree = get_etree(filename)
        provider = {"name": "Test"}

        for parser in registered_feed_parsers.values():
            if parser.can_parse(self.tree):
                self.item = parser.parse(self.tree, provider)[0]


class TextParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture("text.xml")

    def test_instance(self):
        self.assertTrue(self.item)

    def test_parse_id(self):
        self.assertEqual("tag:reuters.com,0000:newsml_L4N0BT5PJ:263518268", self.item.get("guid"))
        self.assertEqual("263518268", self.item.get("version"))
        self.assertEqual(3, self.item.get("priority"))

    def test_parse_item_meta(self):
        self.assertEqual("text", self.item.get("type"))
        self.assertEqual("2013-03-01T15:09:04", self.item.get("versioncreated").isoformat())
        self.assertEqual("2013-03-01T15:09:04", self.item.get("firstcreated").isoformat())
        self.assertEqual("Editorial Note", self.item.get("ednote"))

    def test_parse_content_meta(self):
        self.assertEqual(3, self.item.get("urgency"))
        self.assertEqual("SOCCER-ENGLAND/CHELSEA-BENITEZ", self.item["slugline"])
        self.assertEqual(
            "Soccer-Smiling Benitez pleads for support " "after midweek outburst against opponent",
            self.item["headline"],
        )
        self.assertNotIn("description_text", self.item)
        self.assertNotIn("archive_description", self.item)

    def test_content_set(self):
        self.assertEqual("<p>By Toby Davis</p>", self.item.get("body_html"))
        self.assertEqual(569, self.item.get("word_count"))
        self.assertIsInstance(self.item.get("body_html"), type(""))

    def test_language(self):
        self.assertEqual("en", self.item.get("language"))

    def test_subject(self):
        self.assertEqual(2, len(self.item.get("subject")))
        self.assertIn({"qcode": "15054000", "name": "soccer"}, self.item.get("subject"))

    def test_pubstatus(self):
        self.assertEqual("usable", self.item.get("pubstatus"))


class PictureParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture("picture.xml")

    def test_type(self):
        self.assertEqual("picture", self.item.get("type"))

    def test_content_set(self):
        self.assertEqual(3, len(self.item.get("renditions")))
        self.assertEqual(4, self.item.get("priority"))
        remote = self.item.get("renditions").get("baseImage")
        self.assertTrue(remote)
        self.assertEqual("tag:reuters.com,0000:binary_GM1E9341HD701-BASEIMAGE", remote.get("residRef"))
        self.assertEqual(772617, remote.get("sizeinbytes"))
        self.assertEqual("image/jpeg", remote.get("mimetype"))
        self.assertEqual(
            "http://content.reuters.com/auth-server/content/tag:reuters.com,0000:newsml_GM1E9341HD701:360624134/tag:reuters.com,0000:binary_GM1E9341HD701-BASEIMAGE",
            remote.get("href"),
        )  # noqa
        self.assertIn(
            "A woman casts her ballot inside a polling station in Kenya's town of Gatundu "
            "March 4, 2013. Polling stations opened up to Kenyans on Monday for a tense "
            "presidential election that will test whether the east African nation can repair "
            "its damaged reputation after the tribal blood-letting that followed a "
            "2007 poll.    REUTERS/Marko Djurica (KENYA - Tags: POLITICS ELECTIONS)",
            self.item["description_text"],
        )
        self.assertIn(
            "A woman casts her ballot inside a polling station in Kenya's town of Gatundu "
            "March 4, 2013. Polling stations opened up to Kenyans on Monday for a tense "
            "presidential election that will test whether the east African nation can repair "
            "its damaged reputation after the tribal blood-letting that followed a "
            "2007 poll.    REUTERS/Marko Djurica (KENYA - Tags: POLITICS ELECTIONS)",
            self.item["archive_description"],
        )

    def test_byline(self):
        self.assertEqual("MARKO DJURICA", self.item.get("byline"))

    def test_place(self):
        self.assertEqual(2, len(self.item.get("place")))
        self.assertIn({"name": "NAIROBI"}, self.item["place"])
        self.assertIn({"name": "Kenya"}, self.item["place"])


class SNEPParserTest(ItemTest):
    def setUp(self):
        self.setUpFixture("snep.xml")

    def test_content_set(self):
        self.assertEqual(4, self.item.get("priority"))
        self.assertEqual(2, len(self.item.get("groups")))

        group = self.item.get("groups")[0]
        self.assertTrue(group)
        self.assertEqual("root", group.get("id"))
        self.assertEqual("grpRole:SNEP", group.get("role"))
        self.assertEqual(1, len(group.get("refs")))
        self.assertEqual("main", group.get("refs")[0].get("idRef"))

        group = self.item.get("groups")[1]
        self.assertEqual(10, len(group.get("refs")))
        self.assertEqual("main", group.get("id"))

        ref = group.get("refs")[0]
        self.assertTrue(ref)
        self.assertEqual("tag:reuters.com,0000:newsml_BRE9220HA:15", ref.get("residRef"))
        self.assertEqual("application/vnd.iptc.g2.packageitem+xml", ref.get("contentType"))
        self.assertEqual("icls:composite", ref.get("itemClass"))
        self.assertEqual("At least 15 killed on Kenya coast on election day", ref.get("headline"))



from lxml import etree

from superdesk.utils import merge_dicts_deep

from .base import BasePackageElement


class Spread(BasePackageElement):
    """
    As Adobe's idml specification explains:
      ``
       The Spreads folder contains the XML files representing the spreads in the document.
       Each spread contains all of the page items (rectangles, ellipses, graphic lines, polygons, groups, buttons,
       and text frames) that appear on the pages of the spread.
       The <Spread> element also contains <Page> elements, which contain attributes and elements that relate
       to the pages of the spread. Note that <Page> elements do not contain page items.
       Spreads do not contain text stream content — the <TextFrame> XML elements in the spread refer
       to <Story> elements contained the files in the Stories folder.
      ``
    """

    SPREAD_DEFAULTS = {
        "FlattenerOverride": "Default",
        "ShowMasterItems": "true",
        "PageCount": "1",
        "BindingLocation": "0",
        "AllowPageShuffle": "true",
        "ItemTransform": "1 0 0 1 0 0",
        "PageTransitionType": "None",
        "PageTransitionDirection": "NotApplicable",
        "PageTransitionDuration": "Medium",
    }
    PAGE_DEFAULTS = {
        "AppliedTrapPreset": "TrapPreset/$ID/kDefaultTrapStyleName",
        "AppliedMaster": "n",
        "OverrideList": "",
        "TabOrder": "",
        "GridStartingPoint": "TopOutside",
        "UseMasterGrid": "true",
    }
    MARGINPREFERENCE_DEFAULTS = {
        "ColumnCount": "1",
        "ColumnGutter": "12",
        "Top": "36",
        "Bottom": "36",
        "Left": "36",
        "Right": "36",
        "ColumnDirection": "Horizontal",
        "ColumnsPositions": "0 540",
    }
    TEXTFRAME_DEFAULTS = {"ItemTransform": "1 0 0 1 0 0", "ContentType": "TextType"}
    GEOMETRYPATH_DEFAULTS = {"PathOpen": "false"}
    VERTICAL_MARGIN = 10

    def __init__(self, self_id, document_page_width, document_page_height, attributes=None):
        self.self_id = self_id
        self._spread = None
        self._page = None
        self._document_page_width = document_page_width
        self._document_page_height = document_page_height
        super().__init__(attributes)

    @property
    def filename(self):
        """
        Filename inside IDML package.
        Used as a filename for a file inside zip container.
        :return str: filename
        """
        return "Spreads/Spread_{}.xml".format(self.self_id)

    @property
    def page_inner_height(self):
        return self._document_page_height - self._page_margins["top"] - self._page_margins["bottom"]

    @staticmethod
    def _make_itemtransform_translation(matrix, coordiantes):
        """
        Translate matrix according to provided coordinates.
        :param tuple matrix: initial matrix
        :param tuple coordiantes: typle (x, y)
        :return:
        """
        # x
        matrix[2][0] += coordiantes[0]
        # y
        matrix[2][1] += coordiantes[1]

        return matrix

    @property
    def _initial_itemtransform_matrix(self):
        """
        :return tuple: initial matrix for item transform
        """
        margins = self._page_margins

        return (
            [1, 0],
            [0, 1],
            [
                self._document_page_width / 2 * (-1) + margins["right"],
                self._document_page_height / 2 * (-1) + margins["top"],
            ],
        )

    @property
    def _last_used_pathpoints(self):
        """
        :return list: list of dicts with last used pathpoints
        """
        elements = self._spread.xpath(".//PathPointArray")

        if not elements:
            return None

        pathpointarray = elements[-1]

        return [{k: [float(i) for i in el.attrib[k].split()] for k in el.attrib} for el in pathpointarray]

    @property
    def _last_used_itemtransform(self):
        """
        Return a last (lowest) TextFrame/Rectangle's ItemTransform as a matrix
        :return tuple: last used item transform as a matrix
        """
        elements = self._spread.xpath("*[self::TextFrame|self::Rectangle]")

        if not elements:
            return None

        _transform = [float(i) for i in elements[-1].get("ItemTransform").split()]

        return (_transform[:2], _transform[2:4], _transform[4:])

    @property
    def _page_margins(self):
        """
        :return dict: page margins info
        """
        if self._page is None:
            # TODO create custom exception
            raise Exception("Page does not exist.")

        return {
            "top": float(self._page.find("MarginPreference").attrib["Top"]),
            "bottom": float(self._page.find("MarginPreference").attrib["Bottom"]),
            "left": float(self._page.find("MarginPreference").attrib["Left"]),
            "right": float(self._page.find("MarginPreference").attrib["Right"]),
        }

    def add_page(self, attributes=None):
        """
        Add page to the spread.
        :param dict attributes: custom page attributes
        :return lxml.etree: Page etree instance
        """
        # for now only 1 page per spread is supported
        if self._page is not None:
            return False

        # merge Page attributes
        page_attributes = self.PAGE_DEFAULTS
        if attributes:
            page_attributes = self.merge_attributes(page_attributes, attributes.get("Page", {}))
        # Page
        self._page = etree.SubElement(self._spread, "Page", attrib=page_attributes)
        # merge MarginPreference attributes
        marginpreference_attributes = self.MARGINPREFERENCE_DEFAULTS
        if attributes:
            marginpreference_attributes = self.merge_attributes(
                marginpreference_attributes, attributes.get("MarginPreference", {})
            )
        # MarginPreference
        etree.SubElement(self._page, "MarginPreference", attrib=marginpreference_attributes)

        return self._page

    def has_page(self):
        return self._page is not None

    def place_textframe(self, height, attributes=None):
        """
        Place textframe in the spread. Text frame has link to the story.
        :param int height: text frame height
        :param attributes: custom text frame attributes
        """
        if height < 20:
            height = 20

        modifier = {"TextFrame": {"ItemTransform": self._generate_next_itemtransform()}}

        if attributes:
            attributes = dict(merge_dicts_deep(attributes, modifier))
        else:
            attributes = modifier

        textframe = self._create_textframe(attributes)

        # Properties
        properties = etree.SubElement(textframe, "Properties")
        pathgeometry = self._create_pathgeometry(height)
        properties.append(pathgeometry)

    def check_if_fits(self, height):
        """
        Check if text frame with provided `height` will fit in page.
        :param int height: height to check
        :return bool: True or False
        """
        # 0:0 is at the center of the page when facing pages is off
        highest_possible_y_axis_point = self.page_inner_height / 2
        required_y_axis_point = (
            float(self._generate_next_itemtransform().rsplit(maxsplit=1)[-1]) + height + self.VERTICAL_MARGIN
        )

        return highest_possible_y_axis_point >= required_y_axis_point

    def _build_etree(self):
        self._etree = etree.Element(etree.QName(self.XMLNS_IDPKG, "Spread"), nsmap={"idPkg": self.XMLNS_IDPKG})
        self._etree.set("DOMVersion", self.DOM_VERSION)
        self._add_spread()

    def _add_spread(self):
        """
        Create <Spread..> tag
        :return lxml.etree: Spread etree instance
        """
        # merge Spread attributes
        spread_attributes = self.merge_attributes(self.SPREAD_DEFAULTS, self._attributes.get("Spread", {}))
        spread_attributes.update({"Self": self.self_id})
        # Spread
        self._spread = etree.SubElement(self._etree, "Spread", attrib=spread_attributes)

        return self._spread

    def _create_textframe(self, attributes=None):
        """
        Create <TextFrame..> tag
        :param dict attributes: custom attributes
        :return lxml.etree: TextFrame lxml instance
        """

        # merge TextFrame attributes
        textframe_attributes = self.TEXTFRAME_DEFAULTS
        if attributes:
            textframe_attributes = self.merge_attributes(textframe_attributes, attributes.get("TextFrame", {}))
        # Page
        textframe = etree.SubElement(self._spread, "TextFrame", attrib=textframe_attributes)

        return textframe

    def _create_pathgeometry(self, height):
        """
        Create <PathGeometry..> tag
        :param int height: height
        :return lxml.etree: PathGeometry lxml instance
        """
        # for rectangle frames only

        # PathGeometry
        pathgeometry = etree.Element("PathGeometry")
        # GeometryPath
        geometrypath = etree.SubElement(pathgeometry, "GeometryPath", attrib=self.GEOMETRYPATH_DEFAULTS)
        # PathPointArray
        pathpointarray = etree.SubElement(geometrypath, "PathPointArray")
        # PathPoint(s)
        margins = self._page_margins
        internal_page_width = self._document_page_width - margins["right"] - margins["left"]
        pathpoints_coordinates = [
            # left top
            [0, 0],
            # left bottom
            [0, height],
            # right bottom
            [internal_page_width, height],
            # right top
            [internal_page_width, 0],
        ]

        for coordiantes in pathpoints_coordinates:
            etree.SubElement(
                pathpointarray,
                "PathPoint",
                attrib={
                    "Anchor": "{} {}".format(*coordiantes),
                    "LeftDirection": "{} {}".format(*coordiantes),
                    "RightDirection": "{} {}".format(*coordiantes),
                },
            )

        return pathgeometry

    def _generate_next_itemtransform(self):
        """
        :return str: item transform for the next text frame element
        """
        matrix = self._last_used_itemtransform

        if matrix:
            matrix = self._make_itemtransform_translation(matrix, (0, self.VERTICAL_MARGIN))
            # add frame height
            matrix = self._make_itemtransform_translation(matrix, (0, self._last_used_pathpoints[1]["Anchor"][1]))
        else:
            matrix = self._initial_itemtransform_matrix

        return "{:.0f} {:.0f} {:.0f} {:.0f} {} {}".format(
            matrix[0][0],
            matrix[0][1],
            matrix[1][0],
            matrix[1][1],
            matrix[2][0],
            matrix[2][1],
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from superdesk.resource import Resource


class ContentFilterResource(Resource):
    schema = {
        "name": {"type": "string", "required": True, "nullable": False, "empty": False, "iunique": True},
        "content_filter": {
            "type": "list",
            "schema": {
                "type": "dict",
                "schema": {
                    "expression": {
                        "type": "dict",
                        "schema": {
                            "fc": {"type": "list", "schema": Resource.rel("filter_conditions", True)},
                            "pf": {"type": "list", "schema": Resource.rel("content_filters", True)},
                        },
                    }
                },
            },
        },
        "is_global": {"type": "boolean", "default": False},
        "is_archived_filter": {"type": "boolean", "default": False},
        "api_block": {"type": "boolean", "default": False},
    }

    additional_lookup = {"url": 'regex("[\w,.:-]+")', "field": "name"}

    privileges = {"POST": "content_filters", "PATCH": "content_filters", "DELETE": "content_filters"}

    mongo_indexes = {
        "name_1": ([("name", 1)], {"unique": True}),
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from lxml import etree
from unittest import mock
from textwrap import dedent

from superdesk.tests import TestCase
from superdesk.publish.formatters.nitf_formatter import NITFFormatter
from superdesk.publish.formatters import Formatter
from superdesk.publish import init_app


@mock.patch("superdesk.publish.subscribers.SubscribersService.generate_sequence_number", lambda self, subscriber: 1)
class NitfFormatterTest(TestCase):
    def setUp(self):
        self.formatter = NITFFormatter()
        self.base_formatter = Formatter()
        init_app(self.app)

    def test_append_legal(self):
        article = {"slugline": "Obama Republican Healthc", "flags": {"marked_for_legal": True}}

        slugline = self.base_formatter.append_legal(article)
        self.assertEqual(slugline, "Legal: Obama Republican Healthc")
        slugline = self.base_formatter.append_legal(article, truncate=True)
        self.assertEqual(slugline, "Legal: Obama Republican ")

    def test_append_legal_when_not_legal(self):
        article = {"slugline": "Obama Republican Healthc", "flags": {"marked_for_legal": False}}

        slugline = self.base_formatter.append_legal(article)
        self.assertEqual(slugline, "Obama Republican Healthc")

    def test_formatter(self):
        article = {
            "headline": "test headline",
            "body_html": "<p>test body</p><p>привет</p>",
            "type": "text",
            "priority": "1",
            "_id": "urn:localhost.abc",
            "urgency": 2,
        }

        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        nitf_xml = etree.fromstring(doc)
        self.assertEqual(nitf_xml.find("head/title").text, article["headline"])
        self.assertEqual(nitf_xml.findall("body/body.content/p")[0].text, "test body")
        self.assertEqual(nitf_xml.findall("body/body.content/p")[1].text, "привет")
        self.assertEqual(nitf_xml.find("head/docdata/urgency").get("ed-urg"), "2")

    def test_html2nitf(self):
        html = etree.fromstring(
            dedent(
                """\
            <div>
                <unknown>
                    <p>
                        this should be still here
                    </p>
                </unknown>
                <p style="this='is';some='style'">
                    <strong>this text should be
                        <i>modified</i>
                    </strong>
                    so
                    <span>[this should not be removed]</span>
                    unkown
                    <em unknown_attribute="toto">elements</em>
                    and
                    <a bad_attribute="to_remove">attributes</a>
                    are
                    <h6>удаленный</h6>
                </p>
            </div>
            """
            )
        )

        nitf = self.formatter.html2nitf(html, attr_remove=["style"])

        expected = (
            dedent(
                """\
            <div>
                    <p>
                        this should be still here
                    </p>
                <p>
                    <em class="bold">this text should be
                        <em class="italic">modified</em>
                    </em>
                    so [this should not be removed] unkown
                    <em class="italic">elements</em>
                    and
                    <a>attributes</a>
                    are
                    <hl2>удаленный</hl2>
                </p>
            </div>"""
            )
            .replace("\n", "")
            .replace(" ", "")
        )
        self.assertEqual(etree.tostring(nitf, encoding="unicode").replace("\n", "").replace(" ", ""), expected)

    def test_html2nitf_br(self):
        """Check that <br/> is kept if it is a child of and enrichedText parent element"""
        html = etree.fromstring(
            dedent(
                """\
            <div>
                <br/>the previous tag should be удаленный (but not the text)
                    <p>
                        the following tag <br/> should still be here
                        and the next one <br/> too
                    </p>
            </div>
            """
            )
        )

        nitf = self.formatter.html2nitf(html, attr_remove=["style"])

        expected = dedent(
            """\
            <div>
                the previous tag should be удаленный (but not the text)
                    <p>
                        the following tag <br/> should still be here
                        and the next one <br/> too
                    </p>
            </div>"""
        )
        self.assertEqual(etree.tostring(nitf, encoding="unicode"), expected)

    def test_body_content_br(self):
        article = {
            "_id": "urn:newsml:localhost:2017-05-24T16:56:29.742769:3d1faf62-6f70-4b28-9222-93ec603b7af0",
            "guid": "urn:newsml:localhost:2017-05-24T16:56:29.742769:3d1faf62-6f70-4b28-9222-93ec603b7af0",
            "state": "published",
            "body_html": "<p>Sydney's Lindt Cafe siege hostages endured 17-hours of torture but, in the end, it "
            "may have been a 10-minute delay by police that cost two lives.</p><p>Shortly after 2am "
            "on December 16, 2014, gunman Man Haron Monis fired at escaping hostages.</p><p>That first "
            'shot by Monis&nbsp;should have satisfied a so-called "secondary trigger" prompting '
            "police to storm the Martin Place stronghold with some element of surprise, NSW Coroner "
            'Michael Barnes found on Wednesday.</p><p>"(But) the 10 minutes that lapsed without '
            'decisive action by police was too long," Mr Barnes&nbsp;said in Sydney.</p><p>By the '
            "time police smashed their way into the cafe at 2.14am in a flurry of stun-grenade "
            "explosions, manager Tori Johnson had been forced to his knees and shot in the head."
            "</p><p>Barrister Katrina Dawson was killed by police bullet fragments.</p><p>New police "
            "commissioner Mick Fuller - who was one of the first commanders at the scene - admitted on "
            "Wednesday tactical officers should have gone in earlier.</p><p>Mr Fuller went further than "
            'the coroner when he told the Ten Network: "We probably should have gone in before the '
            'first shot."</p><p>"Early intervention for terrorist incidents is the way forward, '
            'knowing what we know now," he said.</p><p>"But nevertheless it\'s still a very '
            'dangerous tactic and people will potentially still lose their lives."</p><p>Mr Barnes '
            "has made 45 findings on everything from police negotiation strategies to greater legal "
            "protections for frontline officers in terrorist situations.<br></p><p>He lay the blame "
            "for the loss of life squarely at the feet of Monis, but other parties, including prosecution "
            "solicitors and a consulting psychiatrist, copped some criticism.</p><p>The cafe "
            'was "transformed into a prison run by a vicious maniac" that day, Mr Barnes '
            'said.<br></p><p class="">The traditional "contain and negotiate" strategy was '
            "appropriate early on but police failed to adequately reassess taking into account the "
            'headway they were making with Monis.<br></p><p class="">"Sadly, it failed," Mr Barnes '
            'said on Wednesday.<br></p><p class="">"The \'contain and negotiate\' approach may not be '
            "the best ongoing response to a terrorist incident if the offenders believe, whether or not "
            "they survive, their cause will benefit from the publicity generated by a protracted "
            'siege."<br></p><p class="">Mr Fuller said the containment strategy had saved countless '
            "lives over the years and wouldn't be abandoned for non-terrorist incidents.</p>"
            '<p class="">Police mistakenly thought Monis\' backpack could house explosives, '
            'but Mr Barnes noted senior officers were "unduly reluctant" to initiate direct action '
            'plans during siege situations.</p><p class="">The "primary trigger" for such an '
            "assault was identified as the death or serious injury of a hostage - evidence which "
            'disgusted the families of victims and survivors.</p><p class="">The police response was '
            "at times hampered by mishaps.</p><p>Eight calls to a phone number hostages expected would "
            'connect them with negotiators were missed, which was a "significant failure", Mr Barnes '
            "said.</p><p>Commanders and tactical officers received specialist terrorist training but "
            'negotiators received "little, if any", the coroner added.</p><p>The stand-off could not '
            "have eventuated in the first place if Monis had not been granted bail on accessory to murder"
            " and dozens of sexual assault offences.<br></p><p>Mr Barnes found the work of an Office of "
            "the Director of Public Prosecutions solicitor in December 2013 was inadequate, "
            '"erroneously" advising a court Monis didn\'t have to show exceptional circumstances '
            "in arguing for bail.</p><p>Police also made a mistake by issuing Monis with a court "
            "attendance notice for the sexual offences in October 2014 rather than arresting him"
            ".</p><p>Monis was already on bail at the time for a commonwealth offence after he'd "
            "written offensive letters to the families of Australian soldiers killed in the Middle"
            " East.</p><p>States can find it difficult to access commonwealth records, Mr Barnes said,"
            " and he called for that to be remedied.</p><p>Some of the coroner's harshest individual "
            "criticism was reserved for the consultant psychiatrist who advised police.</p>"
            '<p>His "sub-optimal" performance included a belief that Monis was merely grandstanding,'
            " Mr Barnes found.</p><p>The doctor should not have been permitted to advise on negotiation "
            'strategy and he made "erroneous and unrealistic assessments" of what was happening inside '
            "the cafe.</p><p>The psychiatrist's advice was ambiguous and Islamic terrorism was beyond his"
            ' expertise.</p><p>"The police commanders underestimated the threat Monis posed," Mr Barnes'
            " said, in part blaming their reliance upon the psychiatrist's opinion.</p><p>He recommended "
            "a more diverse panel of experts be used in the future.</p><p>The coroner stated police "
            "snipers couldn't have ended the siege despite a 10-minute window where they had clear sight "
            "of a head that could have been Monis.</p><p>Those gunmen were never confident in their legal "
            'justification for a "kill shot" and Mr Barnes suggested their&nbsp;power to use force '
            "should be more clearly defined.</p><p>The coroner did acknowledge that sending tactical "
            "officers into the cafe after their hand was forced was a decision no commander would "
            'ever want to face.</p><p>"The bravery of these officers inspires awe," he said.<br></p>',
            "pubstatus": "usable",
            "type": "text",
            "abstract": "<p>The NSW coroner believes a 10-minute period of inaction by police before the bloody end "
            'of the 2014 Lindt Cafe siege was "too long".</p>',
            "priority": 6,
            "unique_id": 12055427,
            "format": "HTML",
            "genre": [{"qcode": "Wrap", "name": "Wrap"}],
            "word_count": 843,
            "source": "AAP",
            "urgency": 1,
            "subject": [
                {"qcode": "16001000", "parent": "16000000", "name": "act of terror"},
                {"qcode": "02001010", "parent": "02001000", "name": "terrorism"},
            ],
            "flags": {
                "marked_archived_only": False,
                "marked_for_legal": False,
                "marked_for_not_publication": False,
                "marked_for_sms": False,
            },
            "headline": "'Ten minutes was too long': Lindt siege",
            "dateline": {
                "source": "AAP",
                "text": "SYDNEY, May 24 AAP -",
                "located": {
                    "city": "Sydney",
                    "country_code": "AU",
                    "country": "Australia",
                    "dateline": "city",
                    "state_code": "NSW",
                    "state": "New South Wales",
                    "alt_name": "",
                    "tz": "Australia/Sydney",
                    "city_code": "Sydney",
                },
            },
            "anpa_category": [{"qcode": "a", "name": "Australian General News"}],
            "unique_name": "#12055427",
            "place": [
                {
                    "name": "NSW",
                    "qcode": "NSW",
                    "country": "Australia",
                    "world_region": "Oceania",
                    "group": "Australia",
                    "state": "New South Wales",
                }
            ],
            "sign_off": "SN/jmk/jcd/pmu",
            "anpa_take_key": "2nd Wrap (pix/video available)",
            "language": "en",
            "slugline": "Cafe",
            "byline": "Jamie McKinnell",
            "version": 2,
        }

        response = self.formatter.format(article, {})
        nitf_xml = etree.fromstring(response[0][1])
        self.assertEqual(
            etree.tostring(nitf_xml.find("body/body.content/p"), encoding="unicode"),
            "<p>Sydney's Lindt Cafe siege hostages endured 17-hours of torture but, in the end, it may "
            "have been a 10-minute delay by police that cost two lives.</p>\n      ",
        )
        self.assertTrue(nitf_xml.findall("body/body.content/p")[1].text.startswith("Shortly after 2am"))

    def test_html2nitf_br_last(self):
        """Check that last <br/> in a <p> element is removed"""
        html = etree.fromstring(
            dedent(
                """\
            <div>
                    <p>
                        the following tag <br/> should still be here
                    </p>
                    <p>
                        and the next one <br/> too
                    </p>
                    <p>
                        but not the last one:<br/>
                    </p>
            </div>
            """
            )
        )

        nitf = self.formatter.html2nitf(html, attr_remove=["style"])

        expected = dedent(
            """\
            <div>
                    <p>
                        the following tag <br/> should still be here
                    </p>
                    <p>
                        and the next one <br/> too
                    </p>
                    <p>
                        but not the last one:
                    </p>
            </div>"""
        )
        self.assertEqual(etree.tostring(nitf, encoding="unicode"), expected)

    def test_html2nitf_style_cleaning(self):
        """Check that <style> element and style attribute are removed from HTML"""
        html = etree.fromstring(
            dedent(
                """\
            <div>
                <style type="text/css">
                    p { margin-bottom: 0.25cm; line-height: 120%; }a:link {  }
                </style>
                <p style="margin-bottom: 0cm; line-height: 100%">Test bla bla bla</p>
                <p style="margin-bottom: 0cm; line-height: 100%">
                    <br/>
                </p>
                <p style="margin-bottom: 0cm; line-height: 100%">
                    <font face="DejaVu Sans, sans-serif">
                        <font style="font-size: 14pt" size="4">
                            <i>
                                <u>
                                    <b>test</b>
                                </u>
                            </i>
                        </font>
                    </font>
                </p>
                <p style="margin-bottom: 0cm; line-height: 100%">toto</p>
                <p style="margin-bottom: 0cm; line-height: 100%">titi</p>
            </div>
            """
            )
        )

        nitf = self.formatter.html2nitf(html, attr_remove=["style"])

        expected = (
            dedent(
                """\
            <div>
                <p>Test bla bla bla</p>
                <p>
                </p>
                <p>
                    <em class="italic">
                        <em class="underscore">
                            <em class="bold">test</em>
                        </em>
                    </em>
                </p>
                <p>toto</p>
                <p>titi</p>
            </div>"""
            )
            .replace("\n", "")
            .replace(" ", "")
        )
        self.assertEqual(etree.tostring(nitf, encoding="unicode").replace("\n", "").replace(" ", ""), expected)

    def test_table(self):
        html_raw = """
        <div>
        <table>
            <tbody>
                <tr>
                    <td>Table cell 1</td>
                    <td>Table cell 2</td>
                    <td>Table cell 3</td>
                </tr>
                <tr>
                    <td>Table cell 2.1</td>
                    <td>Table cell 2.2</td>
                    <td>Table cell 2.3</td>
                </tr>
                <tr>
                    <td>Table cell 3.1</td>
                    <td>Table cell 3.2</td>
                    <td>Table cell 3.3</td>
                </tr>
            </tbody>
        </table>
        </div>
        """.replace(
            "\n", ""
        ).replace(
            " ", ""
        )
        html = etree.fromstring(html_raw)
        nitf = self.formatter.html2nitf(html)
        self.assertEqual(etree.tostring(nitf, encoding="unicode"), html_raw)

    def test_company_codes(self):
        article = {
            "guid": "tag:aap.com.au:20150613:12345",
            "_current_version": 1,
            "anpa_category": [{"qcode": "f", "name": "Finance"}],
            "source": "AAP",
            "headline": "This is a test headline",
            "byline": "joe",
            "slugline": "slugline",
            "subject": [
                {"qcode": "02011001", "name": "international court or tribunal"},
                {"qcode": "02011002", "name": "extradition"},
            ],
            "anpa_take_key": "take_key",
            "unique_id": "1",
            "body_html": "The story body",
            "type": "text",
            "word_count": "1",
            "priority": "1",
            "_id": "urn:localhost.abc",
            "state": "published",
            "urgency": 2,
            "pubstatus": "usable",
            "dateline": {
                "source": "AAP",
                "text": "Los Angeles, Aug 11 AAP -",
                "located": {
                    "alt_name": "",
                    "state": "California",
                    "city_code": "Los Angeles",
                    "city": "Los Angeles",
                    "dateline": "city",
                    "country_code": "US",
                    "country": "USA",
                    "tz": "America/Los_Angeles",
                    "state_code": "CA",
                },
            },
            "creditline": "sample creditline",
            "keywords": ["traffic"],
            "abstract": "sample abstract",
            "place": [
                {
                    "qcode": "Australia",
                    "name": "Australia",
                    "state": "",
                    "country": "Australia",
                    "world_region": "Oceania",
                }
            ],
            "company_codes": [{"name": "YANCOAL AUSTRALIA LIMITED", "qcode": "YAL", "security_exchange": "ASX"}],
        }

        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        nitf_xml = etree.fromstring(doc)
        company = nitf_xml.find("body/body.head/org")
        self.assertEqual(company.text, "YANCOAL AUSTRALIA LIMITED")
        self.assertEqual(company.attrib.get("idsrc", ""), "ASX")
        self.assertEqual(company.attrib.get("value", ""), "YAL")

    def testNoneAsciNamesContent(self):
        article = {
            "_id": "3",
            "source": "AAP",
            "anpa_category": [{"qcode": "a"}],
            "headline": "This is a test headline",
            "byline": "joe",
            "slugline": "slugline",
            "subject": [{"qcode": "02011001"}],
            "anpa_take_key": "take_key",
            "unique_id": "1",
            "type": "text",
            "body_html": "<p>Томми Mäkinen crashes a Škoda in Äppelbo</p>",
            "word_count": "1",
            "priority": 1,
            "linked_in_packages": [{"package": "package", "package_type": "takes"}],
        }
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        nitf_xml = etree.fromstring(doc)
        self.assertEqual(nitf_xml.find("body/body.content/p").text, "Томми Mäkinen crashes a Škoda in Äppelbo")

    def test_null_genre(self):
        article = {
            "_id": "3",
            "source": "AAP",
            "genre": None,
            "anpa_category": [{"qcode": "a"}],
            "headline": "This is a test headline",
            "byline": "joe",
            "slugline": "slugline",
            "subject": [{"qcode": "02011001"}],
            "anpa_take_key": "take_key",
            "unique_id": "1",
            "type": "text",
            "body_html": "<p>Томми Mäkinen crashes a Škoda in Äppelbo</p>",
            "word_count": "1",
            "priority": 1,
            "linked_in_packages": [{"package": "package", "package_type": "takes"}],
        }
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        self.assertIsNotNone(doc)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import responses
from urllib.parse import urljoin
from superdesk.tests import TestCase
from superdesk.text_checkers import tools
from superdesk.text_checkers import ai
from superdesk.text_checkers.ai.base import registered_ai_services, AIServiceBase
from superdesk.errors import SuperdeskApiError
from superdesk import get_resource_service

ai.AUTO_IMPORT = False
TEST_BASE_URL = "https://something.example.org"


class IMatricsTestCase(TestCase):

    maxDiff = None

    item = {
        "_id": "test_1",
        "guid": "test_1",
        "type": "text",
        "version": 1,
        "body_html": "<p>this is a fake article to test the imatrics service, it should be returning some "
        "interesting tags.</p>",
        "abstract": "<p>abstract</p><p>two lines</p>",
        "headline": "test imatrics",
        "slugline": "test imatrics",
    }

    def setUp(self):
        self.app.config["IMATRICS_BASE_URL"] = TEST_BASE_URL
        self.app.config["IMATRICS_USER"] = "some_user"
        self.app.config["IMATRICS_KEY"] = "some_secret_key"
        self.app.config["IMATRICS_SUBJECT_SCHEME"] = "topics"
        registered_ai_services.clear()
        tools.import_services(self.app, ai.__name__, AIServiceBase)

    @responses.activate
    def test_autotagging(self):
        """Check that autotagging is working"""
        self.app.data.insert(
            "vocabularies",
            [
                {
                    "_id": "topics",
                    "items": [
                        {
                            "name": "superdesk name",
                            "qcode": "20000763",
                            "is_active": True,
                        },
                    ],
                }
            ],
        )

        doc = {
            "service": "imatrics",
            "item": self.item,
        }
        ai_service = get_resource_service("ai")
        api_url = urljoin(TEST_BASE_URL, "article/concept")
        responses.add(
            responses.POST,
            api_url,
            match=[
                responses.json_params_matcher(
                    {
                        "uuid": self.item["guid"],
                        "pubStatus": False,
                        "headline": self.item["headline"],
                        "body": [
                            "this is a fake article to test the imatrics service,"
                            " it should be returning some interesting tags.",
                            "abstract",
                            "two lines",
                        ],
                    }
                )
            ],
            json=[
                {
                    "weight": 1,
                    "geometry": {},
                    "links": [],
                    "title": "IT",
                    "type": "topic",
                    "uuid": "e3c482c0-08a4-3b31-a7f1-e231f1ddffc4",
                },
                {
                    "weight": 0.8387794287831216,
                    "geometry": {},
                    "links": [],
                    "title": "Service",
                    "type": "topic",
                    "uuid": "44f52663-52f9-3836-ac45-ae862fe945a3",
                },
                {
                    "weight": 1,
                    "geometry": {},
                    "links": [{"relationType": "", "id": "medtop:20000763", "source": "IPTC", "uri": "", "url": ""}],
                    "title": "informasjons- og kommunikasjonsteknologi",
                    "type": "category",
                    "uuid": "c8a83204-29e0-3a7f-9a0e-51e76d885f7f",
                },
            ],
        )

        ai_service.create([doc])

        expected = {
            "subject": [
                {
                    "name": "IT",
                    "qcode": "e3c482c0-08a4-3b31-a7f1-e231f1ddffc4",
                    "scheme": "imatrics_topic",
                    "source": "imatrics",
                    "altids": {
                        "imatrics": "e3c482c0-08a4-3b31-a7f1-e231f1ddffc4",
                    },
                },
                {
                    "name": "superdesk name",
                    "qcode": "20000763",
                    "scheme": "topics",
                    "source": "imatrics",
                    "altids": {
                        "imatrics": "c8a83204-29e0-3a7f-9a0e-51e76d885f7f",
                        "medtop": "20000763",
                    },
                },
                {
                    "name": "Service",
                    "qcode": "44f52663-52f9-3836-ac45-ae862fe945a3",
                    "scheme": "imatrics_topic",
                    "source": "imatrics",
                    "altids": {
                        "imatrics": "44f52663-52f9-3836-ac45-ae862fe945a3",
                    },
                },
            ]
        }

        self.assertEqual(doc["analysis"], expected)

    @responses.activate
    def test_search(self):
        """Tag searching is returning tags"""
        doc = {
            "service": "imatrics",
            "operation": "search",
            "data": {"term": "informasjons"},
        }
        ai_data_op_service = get_resource_service("ai_data_op")
        api_url = urljoin(TEST_BASE_URL, "concept/get?operation=title_type")
        responses.add(
            responses.POST,
            api_url,
            json={
                "result": [
                    {
                        "longDescription": "",
                        "pubStatus": True,
                        "aliases": [],
                        "latestVersionTimestamp": "2020-07-14T15:26:26Z",
                        "author": "NTB",
                        "createdTimestamp": "2020-07-14T15:26:26Z",
                        "shortDescription": "title",
                        "broader": "",
                        "title": "informasjons- og kommunikasjonsteknologi",
                        "type": "category",
                        "uuid": "c8a83204-29e0-3a7f-9a0e-51e76d885f7f",
                    },
                    {
                        "longDescription": "",
                        "pubStatus": True,
                        "aliases": [],
                        "latestVersionTimestamp": "2020-07-14T15:26:24Z",
                        "author": "NTB",
                        "createdTimestamp": "2020-07-14T15:26:24Z",
                        "shortDescription": "title",
                        "broader": "",
                        "title": "informasjonsvitenskap",
                        "type": "category",
                        "uuid": "af815add-8456-3226-8177-ea0d8e3011eb",
                    },
                ],
                "response": "Request successful.",
                "error": False,
                "scrollID": "9e7da4cf-541f-36b1-b7a0-aa883a76c04f",
            },
        )
        ai_data_op_service.create([doc])

        expected = {
            "tags": {
                "subject": [
                    {
                        "name": "informasjons- og kommunikasjonsteknologi",
                        "qcode": "c8a83204-29e0-3a7f-9a0e-51e76d885f7f",
                        "scheme": "imatrics_category",
                        "source": "imatrics",
                        "description": "title",
                        "altids": {
                            "imatrics": "c8a83204-29e0-3a7f-9a0e-51e76d885f7f",
                        },
                    },
                    {
                        "name": "informasjonsvitenskap",
                        "qcode": "af815add-8456-3226-8177-ea0d8e3011eb",
                        "scheme": "imatrics_category",
                        "source": "imatrics",
                        "description": "title",
                        "altids": {
                            "imatrics": "af815add-8456-3226-8177-ea0d8e3011eb",
                        },
                    },
                ]
            }
        }

        self.assertEqual(doc["result"], expected)

    @responses.activate
    def test_create(self):
        """Tag can be created"""
        doc = {
            "service": "imatrics",
            "operation": "create",
            "data": {"title": "test_create"},
        }
        ai_data_op_service = get_resource_service("ai_data_op")
        api_url = urljoin(TEST_BASE_URL, "concept/create")
        responses.add(
            responses.POST,
            api_url,
            json={"response": "Concept created with uuid: 6083cb74-77b7-3046-8187-a6333b76b5a4.", "error": False},
        )
        ai_data_op_service.create([doc])
        self.assertEqual(doc["result"], {})

    @responses.activate
    def test_create_fail(self):
        """Tag creation conflict report raise an error"""
        doc = {
            "service": "imatrics",
            "operation": "create",
            "data": {"title": "test_create"},
        }
        ai_data_op_service = get_resource_service("ai_data_op")
        api_url = urljoin(TEST_BASE_URL, "concept/create")
        responses.add(
            responses.POST,
            api_url,
            json={
                "response": "A concept of type topic and title test_create already exists with uuid a2d0ce94-e4b2-3102-"
                "9671-9307b464573c.",
                "error": True,
            },
        )
        with self.assertRaises(SuperdeskApiError) as cm:
            ai_data_op_service.create([doc])

        exc = cm.exception
        self.assertEqual(exc.status_code, 502)

    @responses.activate
    def test_delete(self):
        """Tag can be deleted"""
        doc = {
            "service": "imatrics",
            "operation": "delete",
            "data": {"uuid": "afc7e49d-57d0-34af-b184-b7600af362a9"},
        }
        ai_data_op_service = get_resource_service("ai_data_op")
        api_url = urljoin(TEST_BASE_URL, "concept/delete") + "?uuid=afc7e49d-57d0-34af-b184-b7600af362a9"
        responses.add(
            responses.DELETE,
            api_url,
            json={"error": False},
        )
        ai_data_op_service.create([doc])

        self.assertEqual(doc["result"], {})



from unittest.mock import create_autospec

from superdesk.publish.subscribers import SubscribersService


subscriber_service = create_autospec(SubscribersService)
subscriber_service.generate_sequence_number.return_value = 100


class Resource:
    def __init__(self, service):
        self.service = service


resources = {
    "subscribers": Resource(subscriber_service),
}



from unittest.mock import patch
from superdesk.tests import TestCase
from apps.publish.enqueue.enqueue_service import EnqueueService


class NoTakesEnqueueTestCase(TestCase):
    def setUp(self):
        super().setUp()

        self.product_ids = self.app.data.insert(
            "products",
            [
                {"name": "all"},
            ],
        )

        self.subscriber_ids = self.app.data.insert(
            "subscribers",
            [
                {"name": "digi", "subscriber_type": "digital", "is_targetable": True, "products": self.product_ids},
            ],
        )

        self.desk_ids = self.app.data.insert(
            "desks",
            [
                {"name": "sports"},
            ],
        )

        self.service = EnqueueService()

    def test_resend_no_takes(self):
        doc = {"_id": "test"}
        subscribers = [s for s in self.app.data.find_all("subscribers")]
        subscriber_codes = self.service._get_subscriber_codes(subscribers)
        with patch.object(self.service, "_resend_to_subscribers") as resend:
            with patch.object(self.service, "publish_content_api") as content_api:
                self.service.resend(doc, subscribers)
                resend.assert_called_with(doc, subscribers, subscriber_codes, {})
                content_api.assert_called_with(doc, [])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Generic comments module."""

import superdesk

from .comments import CommentsService, CommentsResource, comments_schema  # noqa
from .user_mentions import on_activity_updated
from .inline_comments import handle_inline_mentions


def init_app(app):
    endpoint_name = "comments"
    service = CommentsService(endpoint_name, backend=superdesk.get_backend())
    CommentsResource(endpoint_name, app=app, service=service)

    app.on_updated_activity -= on_activity_updated
    app.on_updated_activity += on_activity_updated

    superdesk.item_update.connect(handle_inline_mentions)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from werkzeug.datastructures import ImmutableMultiDict
from superdesk.tests import TestCase
from superdesk import get_resource_service
from eve.utils import ParsedRequest


class ConceptItemsServiceTestCase(TestCase):
    concept_items = [
        {
            "name": "Hobbit",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "tolkien"],
            "language": "en",
            "definition_text": "Hobbit is a children's fantasy novel by English author J. R. R. Tolkien.",
            "definition_html": "<p>Hobbit is a children's fantasy novel by English author J. R. R. Tolkien.</p>",
        },
        {
            "name": "Lord of the rings",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "tolkien"],
            "language": "en",
            "definition_text": "Lord of the rings is a children's fantasy novel by English author J. R. R. " "Tolkien.",
            "definition_html": "<b>Lord of the rings is a children's fantasy novel by English author J. R. R. "
            "Tolkien.</b>",
        },
        {
            "name": "Bootstrap: Responsive Web Development",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "Jake Spurlock", "Development"],
            "language": "en",
            "definition_text": "Bootstrap: Responsive Web Development. Discover how easy it is to design killer "
            "interfaces and responsive websites with the Bootstrap framework. ",
            "definition_html": "<p><b>Bootstrap</b>: Responsive Web Development. Discover how easy it is to design "
            "killer interfaces and responsive websites with the Bootstrap framework. </p>",
        },
        {
            "name": "The Little Prince",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "Antoine de Saint-Exupéry"],
            "language": "en",
            "definition_text": "The Little Prince is a novella, the most famous work of French aristocrat, writer, "
            "poet, and pioneering aviator Antoine de Saint-Exupéry.",
            "definition_html": "<p>The Little Prince is a novella, the most famous work of French aristocrat, writer, "
            "poet, and pioneering aviator Antoine de Saint-Exupéry.</p>",
        },
        {
            "name": "the Elegance of the Hedgehog",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "Muriel Barbery"],
            "language": "en",
            "definition_text": "the Elegance of the Hedgehog is a novel about parallels and the concealment of one’s "
            "true passions in life.",
            "definition_html": "<p>the Elegance of the Hedgehog is a novel about parallels and the concealment of "
            "one’s true passions in life.</p>",
        },
        {
            "name": "and then there were none",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "Agatha Christie"],
            "language": "en",
            "definition_text": "and then there were none dame Agatha Mary Clarissa Christie, Lady Mallowan, DBE "
            "(née Miller; 15 September 1890 – 12 January 1976) was an English writer.",
            "definition_html": "<p>and then there were none dame Agatha Mary Clarissa Christie, Lady Mallowan, DBE "
            "(née Miller; 15 September 1890 – 12 January 1976) was an English writer.</p>",
        },
        {
            "name": "A Message to Garcia",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "Elbert Hubbard"],
            "language": "en",
            "definition_text": "A Message to Garcia is a widely distributed essay written by Elbert Hubbard in 1899, "
            "expressing the value of individual initiative and conscientiousness in work.",
            "definition_html": "<p>A Message to Garcia is a widely distributed essay written by Elbert Hubbard in "
            "1899, expressing the value of individual initiative and conscientiousness in work.</p>",
        },
        {
            "name": "Гайдамаки",
            "cpnat_type": "cpnat:abstract",
            "labels": ["book", "Шевченко Тарас Григорович"],
            "language": "en",
            "definition_text": "Гайдамаки — історико-героїчна поема Шевченка, перший український історичний роман у "
            "віршах.",
            "definition_html": "<p><b>Гайдамаки</b> — історико-героїчна поема Шевченка, перший український історичний "
            "роман у віршах.</p>",
        },
    ]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert("concept_items", self.concept_items)

    def test_query_all_items(self):
        service = get_resource_service("concept_items")

        self.assertEqual(len(list(service.get_from_mongo(req=None, lookup={}))), len(self.concept_items))

    def test_query_sort_by_name_case_sensetive(self):
        service = get_resource_service("concept_items")
        names = [
            "A Message to Garcia",
            "Bootstrap: Responsive Web Development",
            "Hobbit",
            "Lord of the rings",
            "The Little Prince",
            "and then there were none",
            "the Elegance of the Hedgehog",
            "Гайдамаки",
        ]

        req = ParsedRequest()
        req.sort = "name"
        cursor = service.get_from_mongo(req=req, lookup={})
        self.assertEqual([i["name"] for i in cursor], names)

        req = ParsedRequest()
        req.sort = "-name"
        names.reverse()
        cursor = service.get_from_mongo(req=req, lookup={})
        self.assertEqual([i["name"] for i in cursor], names)

    def test_query_sort_by_name_case_insensetive(self):
        service = get_resource_service("concept_items")
        names = [
            "A Message to Garcia",
            "and then there were none",
            "Bootstrap: Responsive Web Development",
            "Hobbit",
            "Lord of the rings",
            "the Elegance of the Hedgehog",
            "The Little Prince",
            "Гайдамаки",
        ]

        req = ParsedRequest()
        req.sort = "name"
        req.args = ImmutableMultiDict([("collation", '{"locale": "en", "strength":"1"}')])
        cursor = service.get_from_mongo(req=req, lookup={})
        self.assertEqual([i["name"] for i in cursor], names)

        req = ParsedRequest()
        req.sort = "-name"
        req.args = ImmutableMultiDict([("collation", '{"locale": "en", "strength":"1"}')])
        names.reverse()
        cursor = service.get_from_mongo(req=req, lookup={})
        self.assertEqual([i["name"] for i in cursor], names)

    def test_service_adding_case_insensetive_collation(self):
        service = get_resource_service("concept_items")
        names = [
            "A Message to Garcia",
            "and then there were none",
            "Bootstrap: Responsive Web Development",
            "Hobbit",
            "Lord of the rings",
            "the Elegance of the Hedgehog",
            "The Little Prince",
            "Гайдамаки",
        ]
        definitions = [
            "A Message to Garcia is a widely distributed essay written by Elbert Hubbard in 1899, "
            "expressing the value of individual initiative and conscientiousness in work.",
            "and then there were none dame Agatha Mary Clarissa Christie, Lady Mallowan, DBE "
            "(née Miller; 15 September 1890 – 12 January 1976) was an English writer.",
            "Bootstrap: Responsive Web Development. Discover how easy it is to design killer "
            "interfaces and responsive websites with the Bootstrap framework. ",
            "Hobbit is a children's fantasy novel by English author J. R. R. Tolkien.",
            "Lord of the rings is a children's fantasy novel by English author J. R. R. " "Tolkien.",
            "the Elegance of the Hedgehog is a novel about parallels and the concealment of one’s "
            "true passions in life.",
            "The Little Prince is a novella, the most famous work of French aristocrat, writer, "
            "poet, and pioneering aviator Antoine de Saint-Exupéry.",
            "Гайдамаки — історико-героїчна поема Шевченка, перший український історичний роман у " "віршах.",
        ]

        req = ParsedRequest()
        req.sort = "name"
        cursor = service.get(req=req, lookup={})
        self.assertEqual([i["name"] for i in cursor], names)

        req = ParsedRequest()
        req.sort = "-name"
        names.reverse()
        cursor = service.get(req=req, lookup={})
        self.assertEqual([i["name"] for i in cursor], names)

        req = ParsedRequest()
        req.sort = "definition_text"
        cursor = service.get(req=req, lookup={})
        self.assertEqual([i["definition_text"] for i in cursor], definitions)

        req = ParsedRequest()
        req.sort = "-definition_text"
        definitions.reverse()
        cursor = service.get(req=req, lookup={})
        self.assertEqual([i["definition_text"] for i in cursor], definitions)

    def test_service_use_definition_text_instead_of_definition_html(self):
        service = get_resource_service("concept_items")
        definitions = [
            "A Message to Garcia is a widely distributed essay written by Elbert Hubbard in 1899, "
            "expressing the value of individual initiative and conscientiousness in work.",
            "and then there were none dame Agatha Mary Clarissa Christie, Lady Mallowan, DBE "
            "(née Miller; 15 September 1890 – 12 January 1976) was an English writer.",
            "Bootstrap: Responsive Web Development. Discover how easy it is to design killer "
            "interfaces and responsive websites with the Bootstrap framework. ",
            "Hobbit is a children's fantasy novel by English author J. R. R. Tolkien.",
            "Lord of the rings is a children's fantasy novel by English author J. R. R. " "Tolkien.",
            "the Elegance of the Hedgehog is a novel about parallels and the concealment of one’s "
            "true passions in life.",
            "The Little Prince is a novella, the most famous work of French aristocrat, writer, "
            "poet, and pioneering aviator Antoine de Saint-Exupéry.",
            "Гайдамаки — історико-героїчна поема Шевченка, перший український історичний роман у " "віршах.",
        ]

        req = ParsedRequest()
        req.sort = "definition_html"
        cursor = service.get(req=req, lookup={})
        self.assertEqual([i["definition_text"] for i in cursor], definitions)

        req = ParsedRequest()
        req.sort = "-definition_html"
        definitions.reverse()
        cursor = service.get(req=req, lookup={})
        self.assertEqual([i["definition_text"] for i in cursor], definitions)



import json
from flask import url_for

from superdesk import get_resource_service
from apps.archive.common import ARCHIVE


def test_service_get(prodapi_app_with_data):
    """
    Test fetching items using `archive` service
    :param prodapi_app_with_data: prod api app with filled data
    """

    with prodapi_app_with_data.app_context():
        items_service = get_resource_service(ARCHIVE)

        assert len(list(items_service.get(req=None, lookup={}))) == 7


def test_readonly(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that `archive` endpoint is readonly
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        for method, status in (("get", 200), ("post", 405), ("patch", 405), ("put", 405), ("delete", 405)):
            # we send a request
            resp = getattr(prodapi_app_with_data_client, method)(url_for("archive|resource"))
            # we get a response
            assert resp.status_code == status


def test_excluded_fields(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that fields which are listed as `excluded_fields` are not in the response.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    excluded_fields = {
        "fields_meta",
        "unique_id",
        "family_id",
        "event_id",
        "lock_session",
        "lock_action",
        "lock_time",
        "lock_user",
        "_etag",
        "_type",
        "_updated",
        "_created",
        "_current_version",
        "_links",
    }

    with prodapi_app_with_data.test_request_context():
        # list
        resp = prodapi_app_with_data_client.get(url_for("archive|resource"))
        resp_data = json.loads(resp.data.decode("utf-8"))

        for item in resp_data["_items"]:
            assert len(set(item.keys()) & excluded_fields) == 0

        # details
        item = resp_data["_items"][0]
        resp = prodapi_app_with_data_client.get(
            url_for("archive|item_lookup", _id=item["guid"]),
        )
        resp_data = json.loads(resp.data.decode("utf-8"))

        assert len(set(resp_data.keys()) & excluded_fields) == 0


def test_renditions(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that rendition links are correct.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        # details
        resp = prodapi_app_with_data_client.get(
            url_for(
                "archive|item_lookup",
                _id="urn:newsml:localhost:5000:2019-08-14T15:02:48.032188:d1be79f4-1d08-464a-90ac-90542dde4e90",
            ),
        )
        resp_data = json.loads(resp.data.decode("utf-8"))

        assert resp_data["associations"]["editor_0"]["renditions"] == {
            "baseImage": {
                "height": 933,
                "href": "http://localhost:5500/prodapi/v1/assets/5d553c343031e2855a2e5664.jpg",
                "mimetype": "image/jpeg",
                "poi": {"x": 700, "y": 466},
                "width": 1400,
            },
            "original": {
                "height": 1280,
                "href": "http://localhost:5500/prodapi/v1/assets/5d553c333031e2855a2e5662.jpg",
                "mimetype": "image/jpeg",
                "poi": {"x": 960, "y": 640},
                "width": 1920,
            },
            "thumbnail": {
                "height": 120,
                "href": "http://localhost:5500/prodapi/v1/assets/5d553c343031e2855a2e5666.jpg",
                "mimetype": "image/jpeg",
                "poi": {"x": 90, "y": 60},
                "width": 180,
            },
            "viewImage": {
                "height": 426,
                "href": "http://localhost:5500/prodapi/v1/assets/5d553c343031e2855a2e5668.jpg",
                "mimetype": "image/jpeg",
                "poi": {"x": 320, "y": 213},
                "width": 640,
            },
        }


def test_filters(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Test filtering.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        # specify desk using source
        params = {
            "source": json.dumps(
                {"query": {"filtered": {"filter": {"terms": {"task.desk": ["7777777775ec40943b60e"]}}}}}
            )
        }
        resp = prodapi_app_with_data_client.get(url_for("archive|resource", **params))
        resp_data = json.loads(resp.data.decode("utf-8"))

        assert len(resp_data["_items"]) == 1
        assert resp_data["_items"][0]["guid"] == "tag:localhost:5000:2019:1dbebe89-e808-43f5-a27c-91545dac896f"



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from copy import deepcopy
from flask import request
from flask_babel import _
from eve.utils import config

from apps.archive.usage import update_refs
from apps.archive.archive import SOURCE as ARCHIVE
from apps.content import push_item_move_notification
from superdesk.metadata.utils import item_url
from apps.archive.common import insert_into_versions, fetch_item
from superdesk.metadata.item import INGEST_ID, INGEST_VERSION, FAMILY_ID, ITEM_STATE, CONTENT_STATE, GUID_FIELD
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.resource import Resource, build_custom_hateoas
from superdesk.services import BaseService
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk import get_resource_service
from superdesk.metadata.packages import RESIDREF, REFS, GROUPS
from superdesk.metadata.item import MEDIA_TYPES, ASSOCIATIONS, get_schema

custom_hateoas = {"self": {"title": "Archive", "href": "/archive/{_id}"}}


class FetchResource(Resource):
    endpoint_name = "fetch"
    resource_title = endpoint_name

    schema = get_schema(True)
    schema.update(
        {
            "desk": Resource.rel("desks", False, required=True),
            "stage": Resource.rel("stages", False, nullable=True),
            "macro": {"type": "string"},
            "_links": {"type": "dict"},
        }
    )

    url = "ingest/<{0}:id>/fetch".format(item_url)

    resource_methods = ["POST"]
    item_methods = []

    privileges = {"POST": "fetch"}


class FetchService(BaseService):
    def create(self, docs, **kwargs):
        return self.fetch(docs, id=request.view_args.get("id"), **kwargs)

    def fetch(self, docs, id=None, **kwargs):
        id_of_fetched_items = []

        for doc in docs:
            id_of_item_to_be_fetched = doc.get(config.ID_FIELD) if id is None else id

            desk_id = doc.get("desk")
            stage_id = doc.get("stage")

            ingest_service = get_resource_service("ingest")
            ingest_doc = ingest_service.find_one(req=None, _id=id_of_item_to_be_fetched)

            if not ingest_doc:
                raise SuperdeskApiError.notFoundError(
                    _("Fail to found ingest item with _id: {id}").format(id=id_of_item_to_be_fetched)
                )

            if not is_workflow_state_transition_valid("fetch_from_ingest", ingest_doc[ITEM_STATE]):
                raise InvalidStateTransitionError()

            if doc.get("macro"):  # there is a macro so transform it
                macro_kwargs = kwargs.get("macro_kwargs") or {}
                ingest_doc = get_resource_service("macros").execute_macro(
                    ingest_doc,
                    doc.get("macro"),
                    dest_desk_id=desk_id,
                    dest_stage_id=stage_id,
                    **macro_kwargs,
                )

            dest_doc = fetch_item(
                ingest_doc,
                desk_id,
                stage_id,
                # we might want to change state or target from the macro
                state=ingest_doc[ITEM_STATE]
                if ingest_doc.get(ITEM_STATE) and ingest_doc[ITEM_STATE] != CONTENT_STATE.INGESTED
                else doc.get(ITEM_STATE),
                target=ingest_doc.get("target", doc.get("target")),
            )

            id_of_fetched_items.append(dest_doc[config.ID_FIELD])
            ingest_service.patch(id_of_item_to_be_fetched, {"archived": dest_doc["versioncreated"]})

            dest_doc[FAMILY_ID] = ingest_doc[config.ID_FIELD]
            dest_doc[INGEST_ID] = self.__strip_version_from_guid(ingest_doc[GUID_FIELD], ingest_doc.get("version"))
            dest_doc[INGEST_VERSION] = ingest_doc.get("version")

            self.__fetch_items_in_package(dest_doc, desk_id, stage_id, doc.get(ITEM_STATE, CONTENT_STATE.FETCHED))

            self.__fetch_associated_items(dest_doc, desk_id, stage_id, doc.get(ITEM_STATE, CONTENT_STATE.FETCHED))

            desk = get_resource_service("desks").find_one(req=None, _id=desk_id)
            if desk and desk.get("default_content_profile"):
                dest_doc.setdefault("profile", desk["default_content_profile"])

            if dest_doc.get("type", "text") in MEDIA_TYPES:
                dest_doc["profile"] = None

            update_refs(dest_doc, {})

            get_resource_service(ARCHIVE).post([dest_doc])
            insert_into_versions(doc=dest_doc)
            build_custom_hateoas(custom_hateoas, dest_doc)
            superdesk.item_fetched.send(self, item=dest_doc, ingest_item=ingest_doc)
            doc.update(dest_doc)

        if kwargs.get("notify", True):
            ingest_doc.update({"task": dest_doc.get("task")})
            push_item_move_notification(ingest_doc, doc, "item:fetch")

        return id_of_fetched_items

    def __strip_version_from_guid(self, guid, version):
        """
        Checks if guid contains the version for the ingested item and returns the guid without version
        """
        try:
            if not version:
                return guid

            if guid.endswith(":{}".format(str(version))):
                return guid[: -1 * (len(str(version)) + 1)]
        except Exception:
            return guid

    def __fetch_associated_items(self, doc, desk, stage, state):
        """
        Fetches the associated items of a given document
        """
        for key, item in (doc.get(ASSOCIATIONS) or {}).items():
            if item is not None and item.get("state") == "ingested":
                new_item = deepcopy(item)
                new_item["desk"] = desk
                new_item["stage"] = stage
                new_item["state"] = state
                new_ids = self.fetch([new_item], id=None, notify=False)
                item[config.ID_FIELD] = new_ids[0]

    def __fetch_items_in_package(self, dest_doc, desk, stage, state):
        # Note: macro and target information is not user for package publishing.
        # Needs to looked later when package ingest requirements is clear.
        for ref in [ref for group in dest_doc.get(GROUPS, []) for ref in group.get(REFS, []) if ref.get(RESIDREF)]:
            ref["location"] = ARCHIVE

        refs = [
            {config.ID_FIELD: ref.get(RESIDREF), "desk": desk, "stage": stage, ITEM_STATE: state}
            for group in dest_doc.get(GROUPS, [])
            for ref in group.get(REFS, [])
            if ref.get(RESIDREF)
        ]

        if refs:
            new_ref_guids = self.fetch(refs, id=None, notify=False)
            count = 0
            for ref in [ref for group in dest_doc.get(GROUPS, []) for ref in group.get(REFS, []) if ref.get(RESIDREF)]:
                ref[RESIDREF] = ref[GUID_FIELD] = new_ref_guids[count]
                count += 1


superdesk.workflow_state(CONTENT_STATE.FETCHED)
superdesk.workflow_action(
    name="fetch_from_ingest", include_states=["ingested"], privileges=["ingest", "archive", "fetch"]
)

superdesk.workflow_state("routed")
superdesk.workflow_action(name="route", include_states=["ingested"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import urllib
import requests

from flask import current_app as app
from superdesk.errors import SuperdeskApiError
from flask_babel import _


class AlchemyKeywordsProvider:
    """Keyword provider that user the Alchemy API(http://www.alchemyapi.com/api)"""

    def __init__(self):
        self._http = requests.Session()

    def get_keywords(self, text):
        if not app.config["KEYWORDS_KEY_API"]:
            raise SuperdeskApiError.notFoundError(_("AlchemyAPI key is not set"))

        params = {"apikey": app.config["KEYWORDS_KEY_API"], "outputMode": "json"}

        url = (
            app.config["KEYWORDS_BASE_URL"] + "/text/TextGetRankedNamedEntities" + "?" + urllib.parse.urlencode(params)
        )

        values = {"text": text}

        result = ""
        try:
            result = self._http.post(url, data=values)
        except Exception as ex:
            raise SuperdeskApiError.internalError(_("Fail to connect to Alchemy service"), exception=ex)

        try:
            keywords = result.json()
            return keywords.get("entities", [])
        except Exception as ex:
            raise SuperdeskApiError.internalError(_("Fail to parse the response from Alchemy service"), exception=ex)



import os
import tempfile

from PIL import Image
from superdesk.tests import TestCase
from superdesk.media.renditions import generate_renditions, get_renditions_spec

IMG_PATH = os.path.join(os.path.dirname(__file__), "fixtures", "iphone_gpsinfo_exif.JPG")
BIG_IMG_PATH = os.path.join(
    os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
    "features",
    "steps",
    "fixtures",
    "bike.jpg",
)


class GenerateRenditionsTestCase(TestCase):
    def setUp(self):
        self.app.data.insert(
            "vocabularies",
            [
                {
                    "_id": "crop_sizes",
                    "items": [
                        {"is_active": True, "name": "landscape", "width": 200, "height": 100},
                        {"is_active": True, "name": "portrait", "width": 150, "height": 200},
                    ],
                },
            ],
        )

    def save_binary(self, rendition):
        """Used while debugging to check the picture."""
        binary = self.app.media.get(rendition["media"])
        image = Image.open(binary)
        fd, path = tempfile.mkstemp(".jpg")
        with open(fd, "w") as out:
            image.save(out, "jpeg")
            print("saved", path)

    def test_generate_renditions_custom_crop(self):
        inserted = []
        renditions = get_renditions_spec()
        with open(IMG_PATH, "rb") as original:
            generated = generate_renditions(
                original, "id", inserted, "image", "image/jpeg", renditions, self.app.media.url_for_media
            )

        # thumbnail should keep 4:3 ration
        self.assertIn("thumbnail", generated)
        self.assertEqual(160, generated["thumbnail"]["width"])
        self.assertEqual(120, generated["thumbnail"]["height"])

        # landscape should be cropped to 200x100
        self.assertIn("landscape", generated)
        landscape = generated["landscape"]
        self.assertEqual(200, landscape["width"])
        self.assertEqual(100, landscape["height"])
        self.assertEqual(0, landscape["CropLeft"])
        self.assertEqual(400, landscape["CropRight"])
        self.assertEqual(50, landscape["CropTop"])
        self.assertEqual(250, landscape["CropBottom"])

        # portrait should be cropped to 150x200
        self.assertIn("portrait", generated)
        portrait = generated["portrait"]
        self.assertEqual(150, portrait["width"])
        self.assertEqual(200, portrait["height"])
        self.assertEqual(0, portrait["CropTop"])
        self.assertEqual(300, portrait["CropBottom"])
        self.assertEqual(87, portrait["CropLeft"])
        self.assertEqual(312, portrait["CropRight"])

    def test_generate_renditions_base_image(self):
        inserted = []
        renditions = get_renditions_spec()
        with open(BIG_IMG_PATH, "rb") as original:
            generated = generate_renditions(
                original, "id", inserted, "image", "image/jpeg", renditions, self.app.media.url_for_media
            )

        self.assertIn("landscape", generated)
        landscape = generated["landscape"]
        self.assertEqual(200, landscape["width"])
        self.assertEqual(100, landscape["height"])
        self.assertEqual(0, landscape["CropLeft"])
        self.assertEqual(1200, landscape["CropRight"])
        self.assertEqual(500, landscape["CropTop"])
        self.assertEqual(1100, landscape["CropBottom"])
